SET check_function_bodies = false;
CREATE TABLE public.item(
	id serial NOT NULL,
	type int2 NOT NULL,
	CONSTRAINT item_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.album(
	id serial NOT NULL,
	name text NOT NULL,
	type int2 NOT NULL,
	shared bool NOT NULL DEFAULT false,
	create_time bigint NOT NULL,
	cover int4 NOT NULL DEFAULT 0,
	sort_type int2 NOT NULL DEFAULT 0,
	passphrase_share text,
	CONSTRAINT album_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.share(
	passphrase text NOT NULL,
	allow_operation smallint NOT NULL DEFAULT 0,
	access_permission smallint NOT NULL DEFAULT 0,
	props json NOT NULL DEFAULT '{}',
	CONSTRAINT passphrase_pk PRIMARY KEY (passphrase)
);
CREATE TABLE public.geocoding(
	id serial NOT NULL,
	grouping_key text NOT NULL,
	level_1 int4,
	level_2 int4,
	level_3 int4,
	level_4 int4,
	level_5 int4,
	level_6 int4,
	admin_1 int2,
	admin_2 int2,
	admin_3 int2,
	admin_4 int2,
	admin_5 int2,
	admin_6 int2,
	CONSTRAINT geocoding_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.person(
	id serial NOT NULL,
	name text NOT NULL,
	hidden bool NOT NULL DEFAULT false,
	cover integer,
	CONSTRAINT person_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.config(
	key text NOT NULL,
	value text NOT NULL,
	CONSTRAINT config_key_primary PRIMARY KEY (key)
);
CREATE TABLE public.synonym(
	id serial NOT NULL,
	lang int2 NOT NULL,
	id_concept integer,
	text text NOT NULL,
	CONSTRAINT synonym_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.metadata(
	id_unit integer,
	description text NOT NULL,
	orientation int2 NOT NULL DEFAULT 1,
	focal_length text NOT NULL,
	iso text NOT NULL,
	exposure_time text NOT NULL,
	aperture text NOT NULL,
	lens text NOT NULL,
	device text NOT NULL,
	latitude float8,
	longitude float8
);
CREATE TABLE public.condition_album(
	condition json,
	CONSTRAINT condition_album_pk PRIMARY KEY (id)
) INHERITS(public.album)
;
CREATE TABLE public.normal_album(
	item_count int4,
	start_time bigint,
	end_time bigint,
	version bigint NOT NULL,
	CONSTRAINT normal_album_pk PRIMARY KEY (id)
) INHERITS(public.album)
;
CREATE TABLE public.user_info(
	id serial NOT NULL,
	uid bigint,
	name text NOT NULL,
	config json NOT NULL DEFAULT '{}',
	CONSTRAINT user_id_primary PRIMARY KEY (id),
	CONSTRAINT uid_uk UNIQUE (uid)
);
CREATE TABLE public.group_info(
	id serial NOT NULL,
	gid bigint,
	name text NOT NULL,
	CONSTRAINT group_id_primary PRIMARY KEY (id),
	CONSTRAINT gid_uk UNIQUE (gid)
);
CREATE TABLE public.concept(
	id serial NOT NULL,
	stem text NOT NULL,
	hidden bool NOT NULL DEFAULT false,
	threshold int2,
	parent int2[] NOT NULL,
	CONSTRAINT concept_id_primary PRIMARY KEY (id),
	CONSTRAINT stem_uk UNIQUE (stem)
);
CREATE TABLE public.person_group(
	id serial NOT NULL,
	weight int4,
	feature bytea NOT NULL,
	id_cluster integer,
	CONSTRAINT person_group_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.cluster(
	id serial NOT NULL,
	id_person integer,
	CONSTRAINT cluster_id_primary PRIMARY KEY (id)
);
ALTER TABLE public.cluster ADD CONSTRAINT person_fk FOREIGN KEY (id_person)
REFERENCES public.person (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.synonym ADD CONSTRAINT concept_fk FOREIGN KEY (id_concept)
REFERENCES public.concept (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE TABLE public.face(
	id serial NOT NULL,
	bounding_box json NOT NULL,
	landmark json NOT NULL,
	feature bytea NOT NULL,
	picture oid,
	score integer NOT NULL,
	id_unit integer,
	ref_id_unit integer NOT NULL,
	id_person_group integer,
	id_person integer,
	CONSTRAINT face_id_primary PRIMARY KEY (id)
);
ALTER TABLE public.face ADD CONSTRAINT person_group_fk FOREIGN KEY (id_person_group)
REFERENCES public.person_group (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE TABLE public.unit(
	id serial NOT NULL,
	type int2 NOT NULL DEFAULT 0,
	filename text NOT NULL,
	filesize bigint NOT NULL DEFAULT 0,
	createtime bigint NOT NULL DEFAULT 0,
	mtime bigint NOT NULL DEFAULT 0,
	takentime bigint NOT NULL DEFAULT 0,
	duplicate_hash text NOT NULL,
	content_hash text NOT NULL,
	resolution json NOT NULL,
	index_stage int2 NOT NULL,
	id_drive text NOT NULL,
	place smallint NOT NULL DEFAULT 0,
	version bigint NOT NULL,
	id_item integer NOT NULL,
	id_folder integer NOT NULL,
	id_geocoding integer,
	CONSTRAINT unit_id_primary PRIMARY KEY (id)
);
ALTER TABLE public.unit ADD CONSTRAINT item_fk FOREIGN KEY (id_item)
REFERENCES public.item (id) MATCH FULL
ON DELETE CASCADE ON UPDATE RESTRICT;
CREATE TABLE public.thumbnail(
	type int2 NOT NULL,
	status int2 NOT NULL,
	id_unit integer
);
CREATE TABLE public.folder(
	id serial NOT NULL,
	name text NOT NULL,
	parent int4 NOT NULL,
	CONSTRAINT folder_id_primary PRIMARY KEY (id)
);
ALTER TABLE public.thumbnail ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.metadata ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.metadata ADD CONSTRAINT metadata_uq UNIQUE (id_unit);
ALTER TABLE public.unit ADD CONSTRAINT folder_fk FOREIGN KEY (id_folder)
REFERENCES public.folder (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE TABLE public.many_unit_has_many_concept(
	id_unit integer,
	id_concept integer,
	score integer NOT NULL,
	CONSTRAINT many_unit_has_many_concept_pk PRIMARY KEY (id_unit,id_concept)
);
ALTER TABLE public.many_unit_has_many_concept ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.many_unit_has_many_concept ADD CONSTRAINT concept_fk FOREIGN KEY (id_concept)
REFERENCES public.concept (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX thumbnail_id_type_idx ON public.thumbnail
	USING btree
	(
	  id_unit ASC NULLS LAST,
	  type ASC NULLS LAST
	);
CREATE TABLE public.general_tag(
	id serial NOT NULL,
	name text NOT NULL,
	count integer NOT NULL DEFAULT 0,
	CONSTRAINT general_tag_id_primary PRIMARY KEY (id),
	CONSTRAINT general_name_uk UNIQUE (name)
);
CREATE TABLE public.video_additional(
	id_unit integer,
	duration int8 NOT NULL,
	video_info json NOT NULL,
	audio_info json NOT NULL
);
CREATE TABLE public.geocoding_info(
	id serial NOT NULL,
	lang int2 NOT NULL,
	first_level text NOT NULL,
	second_level text NOT NULL,
	country text NOT NULL,
	id_geocoding integer,
	CONSTRAINT geocoding_info_id_primary PRIMARY KEY (id)
);
ALTER TABLE public.geocoding_info ADD CONSTRAINT geocoding_fk FOREIGN KEY (id_geocoding)
REFERENCES public.geocoding (id) MATCH FULL
ON DELETE SET NULL ON UPDATE CASCADE;
CREATE TABLE public.many_unit_has_many_general_tag(
	id_unit integer,
	id_general_tag integer,
	CONSTRAINT many_unit_has_many_general_tag_pk PRIMARY KEY (id_unit,id_general_tag)
);
ALTER TABLE public.many_unit_has_many_general_tag ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.many_unit_has_many_general_tag ADD CONSTRAINT general_tag_fk FOREIGN KEY (id_general_tag)
REFERENCES public.general_tag (id) MATCH FULL
ON DELETE RESTRICT ON UPDATE CASCADE;
CREATE TABLE public.share_comment(
	id serial NOT NULL,
	comment json NOT NULL,
	passphrase_share text,
	CONSTRAINT share_comment_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.address(
	id serial NOT NULL,
	lang int2 NOT NULL,
	address json NOT NULL,
	id_unit integer,
	CONSTRAINT address_id_primary PRIMARY KEY (id)
);
CREATE TABLE public.version_time(
	version bigserial NOT NULL,
	modified_time bigint NOT NULL,
	CONSTRAINT version_id_primary PRIMARY KEY (version)
);
CREATE INDEX time_idx ON public.version_time
	USING btree
	(
	  modified_time ASC NULLS LAST
	);
CREATE FUNCTION public.update_normal_album_trigger_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
   album_id integer;
   item_id integer;
   is_delete bool;
   album_start_time bigint;
   album_end_time bigint;
   unit_time bigint;
   unit_table text;
   normal_album_table text;
    BEGIN
        unit_table = TG_TABLE_SCHEMA || '.unit';
        normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
        IF (TG_OP = 'INSERT') THEN
            album_id = NEW.id_normal_album;
            item_id  = NEW.id_item;
            is_delete= false;
            EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) + 1 WHERE id =' || album_id;
        ELSIF (TG_OP = 'DELETE') THEN
            album_id = OLD.id_normal_album;
            item_id  = OLD.id_item;
            is_delete= true;
            EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) - 1 WHERE id =' || album_id;
        END IF;
        EXECUTE 'SELECT takentime FROM ' || unit_table || ' WHERE id_item = ' || item_id INTO unit_time;
        EXECUTE 'SELECT COALESCE(start_time, 0), COALESCE(end_time, 0) FROM ' || normal_album_table ||
                ' WHERE id =' || album_id INTO album_start_time, album_end_time;
        IF (album_start_time = 0 OR unit_time < album_start_time) THEN
            EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || unit_time || ' WHERE id = ' || album_id;
        ELSIF (is_delete AND unit_time = album_start_time) THEN
            EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || (SELECT public.get_unit_takentime_by_album(album_id, 'MIN', TG_TABLE_SCHEMA)) || ' WHERE id = ' || album_id;
        END IF;
        IF (album_end_time = 0 OR unit_time > album_end_time) THEN
            EXECUTE 'UPDATE ' || normal_album_table || ' SET end_time = ' || unit_time || ' WHERE id = ' || album_id;
        ELSIF (is_delete AND unit_time = album_end_time) THEN
            EXECUTE 'UPDATE ' || normal_album_table || ' SET end_time = ' || (SELECT public.get_unit_takentime_by_album(album_id, 'MAX', TG_TABLE_SCHEMA)) || ' WHERE id = ' || album_id;
        END IF;
        RETURN NULL;
    END;
$$;
CREATE TABLE public.many_item_has_many_normal_album(
	sequence float8 NOT NULL DEFAULT 0,
	id_item integer,
	id_normal_album integer,
	CONSTRAINT many_item_has_many_normal_album_pk PRIMARY KEY (id_item,id_normal_album)
);
ALTER TABLE public.many_item_has_many_normal_album ADD CONSTRAINT item_fk FOREIGN KEY (id_item)
REFERENCES public.item (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.many_item_has_many_normal_album ADD CONSTRAINT normal_album_fk FOREIGN KEY (id_normal_album)
REFERENCES public.normal_album (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX takentime_idx ON public.unit
	USING btree
	(
	  takentime ASC NULLS LAST
	);
CREATE FUNCTION public.get_unit_takentime_by_album ( id_album int4,  max_or_min text,  schema text)
	RETURNS bigint
	LANGUAGE plpgsql
	STABLE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
    DECLARE
        result  bigint;
        unit_table text;
        many_table text;
    BEGIN
        unit_table = schema || '.unit';
        many_table = schema || '.many_item_has_many_normal_album';
    IF max_or_min = 'MIN' THEN
        EXECUTE 'SELECT takentime FROM ' || unit_table || ' WHERE id_item IN(
            SELECT id_item FROM '|| many_table || ' WHERE id_normal_album = ' || id_album ||
        ') ORDER BY takentime ASC limit 1' INTO STRICT result;
    ELSE
        EXECUTE 'SELECT takentime FROM ' || unit_table || ' WHERE id_item IN(
            SELECT id_item FROM '|| many_table || ' WHERE id_normal_album = ' || id_album ||
        ') ORDER BY takentime DESC limit 1' INTO STRICT result;
    END IF;
    RETURN result;
    EXCEPTION
        WHEN no_data_found THEN
            RETURN 0;
    END;
$$;
CREATE INDEX create_time_idx ON public.album
	USING btree
	(
	  create_time ASC NULLS LAST
	);
CREATE TABLE public.live_additional(
	id_unit integer,
	grouping_key text NOT NULL,
	type int2 NOT NULL
);
CREATE TABLE public.burst_additional(
	id_unit integer,
	grouping_key text NOT NULL,
	sequence int2 NOT NULL,
	major bool
);
CREATE FUNCTION public.delete_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
schema  text;
BEGIN
   schema = 'user_' || OLD.id;
   EXECUTE 'DROP SCHEMA ' || schema || ' CASCADE';
   RETURN NULL;
END
$$;
CREATE TRIGGER update_normal_album_trigger
	AFTER INSERT OR DELETE
	ON public.many_item_has_many_normal_album
	FOR EACH ROW
	EXECUTE PROCEDURE public.update_normal_album_trigger_func();
CREATE FUNCTION public.generate_user_schema ()
	RETURNS text
	LANGUAGE c
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY DEFINER
	COST 1
	AS '/var/packages/SynologyMoments/target/usr/lib/libsynophoto-db-trigger.so', 'generate_user_schema';
CREATE FUNCTION public.create_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
schema_name     text;
schema_sql      text;
t               text;
inherit_table   text[] = '{
    "unit", "video_additional", "live_additional", "burst_additional",
    "item",
    "album", "condition_album", "normal_album",
    "face", "person", "cluster",  "person_group",
    "folder", "general_tag",
    "geocoding", "geocoding_info",  "address",
    "many_share_has_many_group_info",
    "many_share_has_many_user_info",
    "many_unit_has_many_concept",
    "many_unit_has_many_general_tag",
    "many_item_has_many_normal_album",
    "metadata",
    "share", "share_comment",
    "thumbnail",
    "version_time",
    "delete_item", "delete_album",
    "notification",
    "thumb_preview",
    "search_timeline",
    "accessed_share_album"}';
BEGIN
    schema_name = 'user_'|| NEW.id;
    EXECUTE 'CREATE SCHEMA ' || schema_name;
    EXECUTE 'SET LOCAL SCHEMA ' || quote_literal(schema_name);
    SELECT public.generate_user_schema() INTO schema_sql;
    EXECUTE schema_sql;
    FOREACH t IN ARRAY inherit_table
    LOOP
        EXECUTE ' ALTER TABLE '|| t || ' INHERIT public.' || t ;
    END LOOP;
    EXECUTE 'SET LOCAL SCHEMA ' || quote_literal('public');
    RETURN NULL;
END;
$$;
CREATE TRIGGER create_user_schema_trigger
	AFTER INSERT
	ON public.user_info
	FOR EACH ROW
	EXECUTE PROCEDURE public.create_user_schema_func();
CREATE FUNCTION public.address_to_tsvector_immutable_func ( address json)
	RETURNS tsvector
	LANGUAGE plpgsql
	IMMUTABLE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    result tsvector;
    field text;
    value text;
    address_array text[];
    geo text[] = '{
      "country",
      "state",
      "county",
      "city",
      "town",
      "district",
      "village",
      "route",
      "landmark"
    }';
BEGIN
    FOREACH field IN ARRAY geo
    LOOP
        SELECT address::json->field, array_append(address_array, value) INTO value, address_array;
    END LOOP;
    SELECT to_tsvector(array_to_string(address_array, ' ')) INTO result;
    RETURN result;
END
$$;
CREATE INDEX address_idx ON public.address
	USING gin
	(
	  (public.address_to_tsvector_immutable_func(address))
	)	WITH (FASTUPDATE = ON);
CREATE TRIGGER delete_user_schema_trigger
	AFTER DELETE
	ON public.user_info
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_user_schema_func();
CREATE VIEW public.timeline_view
AS
SELECT
 item.id as id_item,
 item.type,
 min(unit.takentime) as takentime,
 array_agg(unit.id) as id_unit,
 unit.place
FROM item , unit  WHERE unit.id_item = item.id GROUP BY item.id, unit.place;
CREATE VIEW public.concept_album_view
AS
SELECT
 item_count, C.stem, id_concept, threshold
FROM
 public.concept as C, (
    SELECT count(U.id_item) as item_count, R.id_concept
    FROM many_unit_has_many_concept as R , unit as U
    WHERE U.id = R.id_unit AND U.place <= 0
    GROUP BY R.id_concept
  ) SS WHERE C.id = SS.id_concept AND item_count >= threshold;
ALTER TABLE public.address ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE VIEW public.geocoding_album_view
AS
 SELECT s.id_geocoding,
    s.item_count,
    album_count_table.count AS album_count,
    geocoding.level_1
   FROM geocoding,
    ( SELECT u.id_geocoding,
            count(u.id_item) AS item_count
           FROM unit u,
            item i
          WHERE u.id_item = i.id AND u.place <= 0
          GROUP BY u.id_geocoding
         HAVING u.id_geocoding IS NOT NULL) s,
    ( SELECT level_1, COUNT(*)
           FROM geocoding
          GROUP BY level_1) album_count_table
  WHERE geocoding.id = s.id_geocoding AND
        geocoding.level_1 = album_count_table.level_1;
CREATE VIEW public.geocoding_timeline_view
AS
SELECT
    item.id as id_item,
    item.type,
    min(unit.takentime) as takentime,
    array_agg(unit.id) as id_unit,
   array_agg(unit.id_geocoding) as id_geocoding,
   unit.place
FROM item , unit WHERE unit.id_item = item.id GROUP BY item.id, unit.place;
CREATE VIEW public.concept_timeline_view
AS
SELECT id_item, item.type, id_unit, id_concept, takentime, place
FROM item, (
SELECT
   U.id_item,
   array_agg(distinct(id_unit)) as id_unit,
   array_agg(distinct(id_concept)) as id_concept,
   min(takentime) as takentime,
   U.place
FROM many_unit_has_many_concept as R, unit as U
WHERE U.id = R.id_unit group by id_item, U.place
) SS WHERE item.id = SS.id_item;
ALTER TABLE public.person_group ADD CONSTRAINT cluster_fk FOREIGN KEY (id_cluster)
REFERENCES public.cluster (id) MATCH FULL
ON DELETE SET NULL ON UPDATE CASCADE;
ALTER TABLE public.face ADD CONSTRAINT person_fk FOREIGN KEY (id_person)
REFERENCES public.person (id) MATCH FULL
ON DELETE SET NULL ON UPDATE CASCADE;
CREATE VIEW public.person_album_view
AS
SELECT
    id_person,
    person.name,
    person.hidden,
    person.cover,
    item_count
FROM person ,(
    SELECT
        face.id_person,
        count(distinct(unit.id_item)) as item_count
    FROM face, unit
    WHERE unit.id = face.id_unit AND unit.place <= 0
    GROUP BY id_person
) SS WHERE SS.id_person = person.id;
CREATE VIEW public.person_timeline_view
AS
SELECT SS.id_item, item.type, SS.id_unit, SS.id_person, SS.takentime, SS.place
FROM  (SELECT
        U.id_item,
        array_agg(DISTINCT(F.id_unit)) as id_unit,
        array_agg(DISTINCT(F.id_person)) as id_person,
        min(takentime) as takentime,
        U.place
    FROM face F, unit U WHERE F.id_unit = U.id
    GROUP BY U.id_item, U.place
) SS, item WHERE item.id = SS.id_item;
CREATE VIEW public.general_tag_album_view
AS
SELECT
  T.id as id_general_tag, T.name, SS.item_count
FROM
  general_tag as T, (
      SELECT count(U.id_item) AS item_count, R.id_general_tag
      FROM many_unit_has_many_general_tag as R, unit as U
      WHERE U.id = R.id_unit AND U.place <= 0
      GROUP BY R.id_general_tag
    ) SS WHERE T.id = SS.id_general_tag;
CREATE VIEW public.general_tag_timeline_view
AS
SELECT item.id as id_item, item.type, id_unit, id_general_tag, takentime, place
FROM
  item,  (
      SELECT
        U.id_item,
        array_agg(distinct(id_unit)) as id_unit,
        array_agg(distinct(id_general_tag)) as id_general_tag,
        min(takentime) as takentime,
        U.place
        FROM many_unit_has_many_general_tag as R, unit as U WHERE U.id = R.id_unit group by id_item, U.place
    ) SS WHERE item.id = SS.id_item;
ALTER TABLE public.unit ADD CONSTRAINT geocoding_fk FOREIGN KEY (id_geocoding)
REFERENCES public.geocoding (id) MATCH FULL
ON DELETE SET NULL ON UPDATE CASCADE;
CREATE TABLE public.administrative(
	id serial NOT NULL,
	value text NOT NULL,
	CONSTRAINT adminstrative_pk PRIMARY KEY (id),
	CONSTRAINT value_uk UNIQUE (value)
);
CREATE TABLE public.index_queue(
	id serial NOT NULL,
	id_user integer NOT NULL,
	id_unit integer NOT NULL,
	type int2 NOT NULL DEFAULT 0,
	status int2 NOT NULL DEFAULT 0,
	task bytea NOT NULL,
	CONSTRAINT index_id_pk PRIMARY KEY (id)
);
CREATE TABLE public.thumb_preview(
	picture oid NOT NULL,
	id_unit integer NOT NULL
);
ALTER TABLE public.thumb_preview ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.thumb_preview ADD CONSTRAINT thumb_preview_uq UNIQUE (id_unit);
CREATE RULE drop_preview_lo_on_delete AS ON DELETE
	TO public.thumb_preview
	DO ALSO (SELECT lo_unlink(OLD.picture));
CREATE RULE drop_preview_lo_on_update AS ON UPDATE
	TO public.thumb_preview
	WHERE NEW.picture != OLD.picture
	DO ALSO (SELECT lo_unlink(OLD.picture));
CREATE RULE drop_face_lo_on_update AS ON UPDATE
	TO public.face
	WHERE NEW.picture != OLD.picture
	DO ALSO (SELECT lo_unlink(OLD.picture));
CREATE RULE drop_face_lo_on_delete AS ON DELETE
	TO public.face
	DO ALSO (SELECT lo_unlink(OLD.picture));
CREATE VIEW public.unit_to_general_tag_view
AS
SELECT
   G.id as id_general_tag, name, R.id_unit
   FROM
   public.general_tag as G
   JOIN public.many_unit_has_many_general_tag as R ON G.id = R.id_general_tag;
CREATE INDEX adress_id_unit_lang_idx ON public.address
	USING btree
	(
	  id ASC NULLS LAST,
	  lang ASC NULLS LAST
	);
CREATE INDEX geocoding_grouping_key_idx ON public.geocoding
	USING btree
	(
	  grouping_key ASC NULLS LAST
	);
CREATE INDEX geocoding_level_1_idx ON public.geocoding
	USING btree
	(
	  level_1 ASC NULLS LAST
	);
CREATE INDEX geocoding_level_2_idx ON public.geocoding
	USING btree
	(
	  level_2 ASC NULLS LAST
	);
CREATE INDEX geocoding_level_3_idx ON public.geocoding
	USING btree
	(
	  level_3 ASC NULLS LAST
	);
CREATE INDEX geocoding_level_4_idx ON public.geocoding
	USING btree
	(
	  level_4 ASC NULLS LAST
	);
CREATE INDEX geocoding_level_5_idx ON public.geocoding
	USING btree
	(
	  level_5 ASC NULLS LAST
	);
CREATE INDEX geocoding_level_6_idx ON public.geocoding
	USING btree
	(
	  level_6 ASC NULLS LAST
	);
CREATE INDEX geocoding_info_id_geocoding_lang_idx ON public.geocoding_info
	USING btree
	(
	  id_geocoding ASC NULLS LAST,
	  lang ASC NULLS LAST
	);
CREATE INDEX unit_id_geocoding_idx ON public.unit
	USING btree
	(
	  id_geocoding ASC NULLS LAST
	);
CREATE INDEX unit_id_item_idx ON public.unit
	USING btree
	(
	  id_item ASC NULLS LAST
	);
CREATE INDEX unit_id_folder_filename_idx ON public.unit
	USING btree
	(
	  id_folder ASC NULLS LAST,
	  filename ASC NULLS LAST
	);
CREATE INDEX item_type_idx ON public.item
	USING btree
	(
	  type ASC NULLS LAST
	);
CREATE INDEX person_group_id_cluster_idx ON public.person_group
	USING btree
	(
	  id_cluster ASC NULLS LAST
	);
CREATE INDEX cluster_id_person_idx ON public.cluster
	USING btree
	(
	  id_person ASC NULLS LAST
	);
CREATE INDEX face_id_unit_idx ON public.face
	USING btree
	(
	  id_unit ASC NULLS LAST
	);
CREATE INDEX unit_id_item_takentime_idx ON public.unit
	USING btree
	(
	  id_item ASC NULLS LAST,
	  takentime ASC NULLS LAST
	);
CREATE TABLE public.delete_item(
	id_item integer NOT NULL,
	version bigint NOT NULL
);
CREATE TABLE public.delete_album(
	id_normal_album integer NOT NULL,
	version bigint NOT NULL
);
CREATE FUNCTION public.update_tag_ref_count_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
   tag_id integer;
   check_id integer;
   ref_count integer;
   general_tag_table text;
    BEGIN
        general_tag_table = TG_TABLE_SCHEMA || '.general_tag';
        IF (TG_OP = 'INSERT') THEN
            tag_id = NEW.id_general_tag;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) + 1 WHERE id = ' || tag_id;
        ELSIF (TG_OP = 'UPDATE') THEN
             tag_id = NEW.id_general_tag;
             check_id = OLD.id_general_tag;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) + 1 WHERE id = ' || tag_id;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) - 1 WHERE id = ' || check_id;
        ELSIF (TG_OP = 'DELETE') THEN
            check_id = OLD.id_general_tag;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) - 1 WHERE id = ' || check_id;
        END IF;
        IF check_id IS NOT NULL THEN
           EXECUTE 'SELECT count  FROM '|| general_tag_table || ' WHERE id = ' || check_id INTO ref_count;
           IF (ref_count  <=0 ) THEN
            EXECUTE 'DELETE FROM ' || general_tag_table || ' WHERE id = ' || check_id;
            END IF;
        END IF;
        RETURN NULL;
    END;
$$;
CREATE TRIGGER update_tag_ref_trigger
	AFTER INSERT OR DELETE OR UPDATE
	ON public.many_unit_has_many_general_tag
	FOR EACH ROW
	EXECUTE PROCEDURE public.update_tag_ref_count_func();
CREATE TABLE public.notification(
	id serial NOT NULL,
	id_user integer NOT NULL,
	props json NOT NULL DEFAULT '{}',
	createtime bigint NOT NULL,
	is_read bool NOT NULL DEFAULT false,
	CONSTRAINT notification_id_pk PRIMARY KEY (id)
);
CREATE INDEX notification_createtime_idx ON public.notification
	USING btree
	(
	  createtime ASC NULLS LAST
	);
CREATE VIEW public.drive_geocoding_album_view
AS
 SELECT s.id_geocoding,
    s.item_count,
    album_count_table.count AS album_count,
    geocoding.level_1
   FROM geocoding,
    ( SELECT u.id_geocoding,
            count(u.id_item) AS item_count
           FROM unit u,
            item i
          WHERE u.id_item = i.id AND u.place <= 1
          GROUP BY u.id_geocoding
         HAVING u.id_geocoding IS NOT NULL) s,
    ( SELECT level_1, COUNT(*)
           FROM geocoding
          GROUP BY level_1) album_count_table
  WHERE geocoding.id = s.id_geocoding AND
        geocoding.level_1 = album_count_table.level_1;
CREATE VIEW public.drive_concept_album_view
AS
SELECT
 item_count, C.stem, id_concept, threshold
FROM
 public.concept as C, (
    SELECT count(U.id_item) as item_count, R.id_concept
    FROM many_unit_has_many_concept as R , unit as U
    WHERE U.id = R.id_unit AND U.place <= 1
    GROUP BY R.id_concept
  ) SS WHERE C.id = SS.id_concept AND item_count >= threshold;
CREATE VIEW public.drive_person_album_view
AS
SELECT
    id_person,
    person.name,
    person.hidden,
    person.cover,
    item_count
FROM person ,(
    SELECT
        face.id_person,
        count(distinct(unit.id_item)) as item_count
    FROM face, unit
    WHERE unit.id = face.id_unit AND unit.place <= 1
    GROUP BY id_person
) SS WHERE SS.id_person = person.id;
CREATE VIEW public.drive_general_tag_album_view
AS
SELECT
  T.id as id_general_tag, T.name, SS.item_count
FROM
  general_tag as T, (
      SELECT count(U.id_item) AS item_count, R.id_general_tag
      FROM many_unit_has_many_general_tag as R, unit as U
      WHERE U.id = R.id_unit AND U.place <= 1
      GROUP BY R.id_general_tag
    ) SS WHERE T.id = SS.id_general_tag;
CREATE FUNCTION public.notification_rotate_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    count integer;
    ctime bigint;
	notification_table text;
BEGIN
	notification_table = TG_TABLE_SCHEMA || '.notification';
    EXECUTE 'SELECT count(*) FROM ' || notification_table INTO count;
    IF (count > 200) THEN
          EXECUTE 'SELECT createtime FROM ' || notification_table || ' ORDER BY createtime DESC OFFSET 200 LIMIT 1' INTO ctime;
          EXECUTE 'DELETE FROM ' || notification_table || ' WHERE createtime <= ' || ctime;
    END IF;
    RETURN NULL;
END;
$$;
CREATE TRIGGER notification_rotate_trigger
	AFTER INSERT
	ON public.notification
	FOR EACH ROW
	EXECUTE PROCEDURE public.notification_rotate_func();
CREATE INDEX index_queue_user_unit_type_uk ON public.index_queue
	USING btree
	(
	  id_user ASC NULLS LAST,
	  id_unit ASC NULLS LAST,
	  type ASC NULLS LAST
	);
CREATE INDEX index_queue_status_id_uk ON public.index_queue
	USING btree
	(
	  status ASC NULLS LAST,
	  type ASC NULLS LAST,
	  id ASC NULLS LAST
	);
CREATE INDEX unit_id_item_id_geocoding_idx ON public.unit
	USING btree
	(
	  id_item ASC NULLS LAST,
	  id_geocoding ASC NULLS LAST
	);
ALTER TABLE public.video_additional ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.video_additional ADD CONSTRAINT video_additional_uq UNIQUE (id_unit);
ALTER TABLE public.live_additional ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.live_additional ADD CONSTRAINT live_additional_uq UNIQUE (id_unit);
ALTER TABLE public.burst_additional ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.burst_additional ADD CONSTRAINT burst_additional_uq UNIQUE (id_unit);
CREATE VIEW public.video
AS
SELECT
   video_unit.id,
   video_unit.type,
   video_unit.filename,
   video_unit.filesize,
   video_unit.createtime,
   video_unit.takentime,
   video_unit.duplicate_hash,
   video_unit.content_hash,
   video_unit.resolution,
   video_unit.index_stage,
   video_unit.id_drive,
   video_unit.place,
   video_unit.version,
   video_unit.id_item,
   video_unit.id_folder,
   video_unit.id_geocoding,
   video_additional.duration,
   video_additional.video_info,
   video_additional.audio_info
FROM
   video_additional,
   (SELECT * FROM unit WHERE type = 1) AS video_unit
WHERE
   video_unit.id = video_additional.id_unit;
CREATE VIEW public.burst
AS
SELECT
   burst_unit.id,
   burst_unit.type,
   burst_unit.filename,
   burst_unit.filesize,
   burst_unit.createtime,
   burst_unit.takentime,
   burst_unit.duplicate_hash,
   burst_unit.content_hash,
   burst_unit.resolution,
   burst_unit.index_stage,
   burst_unit.id_drive,
   burst_unit.place,
   burst_unit.version,
   burst_unit.id_item,
   burst_unit.id_folder,
   burst_unit.id_geocoding,
   burst_additional.grouping_key,
   burst_additional.sequence,
   burst_additional.major
FROM
   burst_additional,
   (SELECT * FROM unit WHERE type = 2) AS burst_unit
WHERE
   burst_unit.id = burst_additional.id_unit;
CREATE VIEW public.live
AS
SELECT
   live_unit.id,
   live_unit.type,
   live_unit.filename,
   live_unit.filesize,
   live_unit.createtime,
   live_unit.takentime,
   live_unit.duplicate_hash,
   live_unit.content_hash,
   live_unit.resolution,
   live_unit.index_stage,
   live_unit.id_drive,
   live_unit.place,
   live_unit.version,
   live_unit.id_item,
   live_unit.id_folder,
   live_unit.id_geocoding,
   live_additional.grouping_key,
   live_additional.type AS live_type
FROM
   live_additional,
   (SELECT * FROM unit WHERE type = 3) AS live_unit
WHERE
   live_unit.id = live_additional.id_unit;
CREATE FUNCTION public.delete_normal_album_after_trigger_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    next_version bigint;
    delete_album_table text;
    share_table text;
    BEGIN
        delete_album_table = TG_TABLE_SCHEMA || '.delete_album';
        share_table = TG_TABLE_SCHEMA || '.share';
        IF (TG_OP = 'DELETE') THEN
           SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO next_version;
           EXECUTE 'INSERT INTO ' || delete_album_table || ' (id_normal_album, version)
            VALUES ( '|| OLD.id || ', ' || next_version || ')';
           IF OLD.passphrase_share IS NOT NULL THEN
               EXECUTE 'DELETE FROM '|| share_table || ' WHERE passphrase = ' || OLD.passphrase_share;
           END IF;
        END IF;
        RETURN NULL;
    END;
$$;
CREATE TRIGGER normal_delete_cascade_to_share_trigger
	AFTER DELETE OR UPDATE
	ON public.normal_album
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_normal_album_after_trigger_func();
CREATE TRIGGER condition_delete_cascade_to_share_trigger
	AFTER DELETE OR UPDATE
	ON public.condition_album
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_normal_album_after_trigger_func();
CREATE FUNCTION public.share_check_album_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
   album_id integer;
   album_table text;
    BEGIN
        album_table = TG_TABLE_SCHEMA || '.album';
        IF (TG_OP = 'INSERT' OR TG_OP = 'UPDATE') THEN
           EXECUTE 'SELECT id FROM ' || album_table || ' WHERE id = ' || NEW.id_album INTO album_id;
           IF album_id IS NULL THEN
              RAISE EXCEPTION 'Album id % does not exist', NEW.id_album;
           END IF;
        END IF;
        RETURN NULL;
    END;
$$;
CREATE TRIGGER check_album_trigger
	AFTER INSERT OR UPDATE
	ON public.share
	FOR EACH ROW
	EXECUTE PROCEDURE public.share_check_album_func();
CREATE INDEX unit_type_idx ON public.unit
	USING btree
	(
	  type ASC NULLS LAST
	);
CREATE FUNCTION public.update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    new_version bigint;
BEGIN
    IF (TG_OP = 'INSERT') THEN
        SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
        NEW.version = new_version;
        RETURN NEW;
    ELSIF (TG_OP = 'UPDATE') THEN
        IF (NEW.index_stage != OLD.index_stage OR NEW.filename != OLD.filename OR NEW.id_folder != OLD.id_folder OR NEW.place != OLD.place OR NEW.takentime != OLD.takentime) THEN
           SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
           NEW.version = new_version;
        END IF;
        RETURN NEW;
    END IF;
    RETURN NULL;
END
$$;
CREATE TRIGGER update_unit_version_trigger
	BEFORE INSERT OR UPDATE
	ON public.unit
	FOR EACH ROW
	EXECUTE PROCEDURE public.update_unit_version_func();
CREATE FUNCTION public.get_next_version_func ( schema text)
	RETURNS bigint
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  next_version bigint;
  version_table text;
BEGIN
  version_table = schema || '.version_time';
  EXECUTE 'INSERT INTO ' || version_table || '(modified_time)
    SELECT round(extract(epoch FROM now()) * 1000) RETURNING version ' INTO next_version;
  RETURN next_version;
END
$$;
CREATE FUNCTION public.update_normal_album_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  next_version bigint;
BEGIN
  IF (TG_OP = 'INSERT' or (TG_OP = 'UPDATE' AND (NEW.item_count != OLD.item_count OR NEW.cover != OLD.cover OR NEW.name != OLD.name OR NEW.shared != OLD.shared OR NEW.sort_type != OLD.sort_type))) THEN
       SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO next_version;
       NEW.version = next_version;
  END IF;
  RETURN NEW;
END
$$;
CREATE TRIGGER update_normal_album_version_trigger
	BEFORE INSERT OR UPDATE
	ON public.normal_album
	FOR EACH ROW
	EXECUTE PROCEDURE public.update_normal_album_version_func();
CREATE FUNCTION public.face_update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  IF (TG_OP = 'UPDATE' AND NEW.id_person != OLD.id_person) THEN
       SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
       EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || NEW.ref_id_unit;
  END IF;
  RETURN NULL;
END
$$;
CREATE TRIGGER face_update_unit_version_trigger
	AFTER UPDATE
	ON public.face
	FOR EACH ROW
	EXECUTE PROCEDURE public.face_update_unit_version_func();
CREATE FUNCTION public.metadata_update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  IF (TG_OP = 'UPDATE') THEN
    IF (NEW.description != OLD.description OR NEW.orientation != OLD.orientation) THEN
       SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
       EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || NEW.id_unit;
    END IF;
  END IF;
  RETURN NULL;
END
$$;
CREATE TRIGGER metadata_update_unit_version_trigger
	AFTER UPDATE
	ON public.metadata
	FOR EACH ROW
	EXECUTE PROCEDURE public.metadata_update_unit_version_func();
CREATE FUNCTION public.tag_update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
  IF (TG_OP = 'INSERT') THEN
     EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || NEW.id_unit;
  ELSIF (TG_OP = 'DELETE') THEN
     EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || OLD.id_unit;
  END IF;
  RETURN NULL;
END
$$;
CREATE TRIGGER tag_update_unit_version_trigger
	AFTER INSERT OR DELETE
	ON public.many_unit_has_many_general_tag
	FOR EACH ROW
	EXECUTE PROCEDURE public.tag_update_unit_version_func();
CREATE FUNCTION public.delete_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  count integer;
  new_version bigint;
  unit_table text;
  delete_item_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  IF (TG_OP = 'DELETE') THEN
    EXECUTE 'SELECT count(*) FROM ' || unit_table || ' WHERE id_item = ' || OLD.id_item INTO count;
    SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
    IF (count > 0) THEN
      EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id_item = ' || OLD.id_item;
    ELSE
      EXECUTE 'INSERT INTO ' || delete_item_table || '(id_item, version)
        VALUES (' || OLD.id_item || ', ' || new_version || ' )';
    END IF;
  END IF;
  RETURN NULL;
END
$$;
CREATE TRIGGER update_unit_after_delete_trigger
	AFTER DELETE
	ON public.unit
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_unit_version_func();
CREATE TABLE public.search_timeline(
	id_item integer NOT NULL,
	type int2 NOT NULL DEFAULT 0,
	takentime bigint NOT NULL DEFAULT 0,
	id_unit integer[] NOT NULL,
	place smallint NOT NULL DEFAULT 0,
	CONSTRAINT search_timeline_pk PRIMARY KEY (id_item)
);
CREATE INDEX search_timeline_idx ON public.search_timeline
	USING btree
	(
	  takentime ASC NULLS LAST
	);
CREATE TABLE public.accessed_share_album(
	passphrase text NOT NULL,
	access_time bigint NOT NULL,
	CONSTRAINT accessed_share_album_pk PRIMARY KEY (passphrase)
);
CREATE FUNCTION public.accessed_share_album_rotate_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    count integer;
    atime bigint;
    accessed_share_album text;
BEGIN
    accessed_share_album = TG_TABLE_SCHEMA || '.accessed_share_album';
    EXECUTE 'SELECT count(*) FROM '|| accessed_share_album INTO count;
    IF (count > 500) THEN
          EXECUTE 'SELECT access_time FROM '|| accessed_share_album ||'  ORDER BY access_time DESC OFFSET 500 LIMIT 1 ' INTO atime;
          EXECUTE 'DELETE FROM '|| accessed_share_album || ' WHERE access_time <= ' || atime;
    END IF;
    RETURN NULL;
END;
$$;
CREATE TRIGGER accessed_share_album_rotate_trigger
	AFTER INSERT
	ON public.accessed_share_album
	FOR EACH ROW
	EXECUTE PROCEDURE public.accessed_share_album_rotate_func();
CREATE FUNCTION public.delete_share_cascade_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  key text;
BEGIN
  key = OLD.passphrase;
  EXECUTE 'DELETE FROM access_share_album WHERE passphrase = ' || quote_literal(key);
  RETURN NULL;
END
$$;
CREATE TRIGGER delete_share_cascade_trigger
	AFTER DELETE
	ON public.share
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_share_cascade_func();
CREATE TABLE public.many_share_has_many_group_info(
	passphrase_share text,
	id_group_info integer,
	create_time bigint NOT NULL,
	CONSTRAINT many_share_has_many_group_info_pk PRIMARY KEY (passphrase_share,id_group_info)
);
ALTER TABLE public.many_share_has_many_group_info ADD CONSTRAINT share_fk FOREIGN KEY (passphrase_share)
REFERENCES public.share (passphrase) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.many_share_has_many_group_info ADD CONSTRAINT group_info_fk FOREIGN KEY (id_group_info)
REFERENCES public.group_info (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.share_comment ADD CONSTRAINT share_fk FOREIGN KEY (passphrase_share)
REFERENCES public.share (passphrase) MATCH FULL
ON DELETE CASCADE ON UPDATE RESTRICT;
ALTER TABLE public.album ADD CONSTRAINT share_fk FOREIGN KEY (passphrase_share)
REFERENCES public.share (passphrase) MATCH FULL
ON DELETE SET NULL ON UPDATE CASCADE;
CREATE TABLE public.many_share_has_many_user_info(
	passphrase_share text,
	id_user_info integer,
	create_time bigint NOT NULL,
	CONSTRAINT many_share_has_many_user_info_pk PRIMARY KEY (passphrase_share,id_user_info)
);
ALTER TABLE public.many_share_has_many_user_info ADD CONSTRAINT share_fk FOREIGN KEY (passphrase_share)
REFERENCES public.share (passphrase) MATCH FULL
ON DELETE RESTRICT ON UPDATE CASCADE;
ALTER TABLE public.many_share_has_many_user_info ADD CONSTRAINT user_info_fk FOREIGN KEY (id_user_info)
REFERENCES public.user_info (id) MATCH FULL
ON DELETE RESTRICT ON UPDATE CASCADE;
ALTER TABLE public.geocoding ADD CONSTRAINT level_1_fk FOREIGN KEY (level_1)
REFERENCES public.administrative (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT;
ALTER TABLE public.geocoding ADD CONSTRAINT level_2_fk FOREIGN KEY (level_2)
REFERENCES public.administrative (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT;
ALTER TABLE public.geocoding ADD CONSTRAINT level_3_fk FOREIGN KEY (level_3)
REFERENCES public.administrative (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT;
ALTER TABLE public.geocoding ADD CONSTRAINT level_4_fk FOREIGN KEY (level_4)
REFERENCES public.administrative (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT;
ALTER TABLE public.geocoding ADD CONSTRAINT level_5_fk FOREIGN KEY (level_5)
REFERENCES public.administrative (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT;
ALTER TABLE public.geocoding ADD CONSTRAINT level_6_fk FOREIGN KEY (level_6)
REFERENCES public.administrative (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT;
ALTER TABLE public.person ADD CONSTRAINT cover_id_face FOREIGN KEY (cover)
REFERENCES public.face (id) MATCH FULL
ON DELETE SET NULL ON UPDATE RESTRICT DEFERRABLE INITIALLY IMMEDIATE;
ALTER TABLE public.face ADD CONSTRAINT person_unit_fk FOREIGN KEY (ref_id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.folder ADD CONSTRAINT folder_parent_fk FOREIGN KEY (parent)
REFERENCES public.folder (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.notification ADD CONSTRAINT notification_id_user_fk FOREIGN KEY (id_user)
REFERENCES public.user_info (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE VIEW public.level_1_timeline_view
AS
SELECT
u.id AS id_unit, u.id_item, u.takentime, u.place, geocoding.level_1 AS level, geocoding.admin_1 AS admin
FROM public.unit u
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_1_person_timeline_view
AS
SELECT
p.id_unit, u.id_item, u.takentime, u.place, geocoding.level_1 AS level, geocoding.admin_1 AS admin, p.id_person
FROM (
SELECT id_unit, array_agg(DISTINCT id_person) AS id_person
FROM public.face GROUP BY id_unit
) p
LEFT JOIN public.unit u ON p.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_1_concept_timeline_view
AS
SELECT
c.id_unit, u.id_item, u.takentime, u.place, geocoding.level_1 AS level, geocoding.admin_1 AS admin, c.id_concept
FROM (
SELECT id_unit, array_agg(DISTINCT id_concept) AS id_concept
FROM public.many_unit_has_many_concept GROUP BY id_unit
) c
LEFT JOIN public.unit u ON c.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_1_general_tag_timeline_view
AS
SELECT
g.id_unit, u.id_item, u.takentime, u.place, geocoding.level_1 AS level, geocoding.admin_1 AS admin, g.id_general_tag
FROM (
SELECT id_unit, array_agg(DISTINCT id_general_tag) AS id_general_tag
FROM public.many_unit_has_many_general_tag GROUP BY id_unit
) g
LEFT JOIN public.unit u ON g.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_2_timeline_view
AS
SELECT
u.id AS id_unit, u.id_item, u.takentime, u.place, geocoding.level_2 AS level, geocoding.admin_2 AS admin
FROM public.unit u
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_2_person_timeline_view
AS
SELECT
p.id_unit, u.id_item, u.takentime, u.place, geocoding.level_2 AS level, geocoding.admin_2 AS admin, p.id_person
FROM (
SELECT id_unit, array_agg(DISTINCT id_person) AS id_person
FROM public.face GROUP BY id_unit
) p
LEFT JOIN public.unit u ON p.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_2_concept_timeline_view
AS
SELECT
c.id_unit, u.id_item, u.takentime, u.place, geocoding.level_2 AS level, geocoding.admin_2 AS admin, c.id_concept
FROM (
SELECT id_unit, array_agg(DISTINCT id_concept) AS id_concept
FROM public.many_unit_has_many_concept GROUP BY id_unit
) c
LEFT JOIN public.unit u ON c.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_2_general_tag_timeline_view
AS
SELECT
g.id_unit, u.id_item, u.takentime, u.place, geocoding.level_2 AS level, geocoding.admin_2 AS admin, g.id_general_tag
FROM (
SELECT id_unit, array_agg(DISTINCT id_general_tag) AS id_general_tag
FROM public.many_unit_has_many_general_tag GROUP BY id_unit
) g
LEFT JOIN public.unit u ON g.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_3_timeline_view
AS
SELECT
u.id AS id_unit, u.id_item, u.takentime, u.place, geocoding.level_3 AS level, geocoding.admin_3 AS admin
FROM public.unit u
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_3_person_timeline_view
AS
SELECT
p.id_unit, u.id_item, u.takentime, u.place, geocoding.level_3 AS level, geocoding.admin_3 AS admin, p.id_person
FROM (
SELECT id_unit, array_agg(DISTINCT id_person) AS id_person
FROM public.face GROUP BY id_unit
) p
LEFT JOIN public.unit u ON p.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_3_concept_timeline_view
AS
SELECT
c.id_unit, u.id_item, u.takentime, u.place, geocoding.level_3 AS level, geocoding.admin_3 AS admin, c.id_concept
FROM (
SELECT id_unit, array_agg(DISTINCT id_concept) AS id_concept
FROM public.many_unit_has_many_concept GROUP BY id_unit
) c
LEFT JOIN public.unit u ON c.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_3_general_tag_timeline_view
AS
SELECT
g.id_unit, u.id_item, u.takentime, u.place, geocoding.level_3 AS level, geocoding.admin_3 AS admin, g.id_general_tag
FROM (
SELECT id_unit, array_agg(DISTINCT id_general_tag) AS id_general_tag
FROM public.many_unit_has_many_general_tag GROUP BY id_unit
) g
LEFT JOIN public.unit u ON g.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_4_timeline_view
AS
SELECT
u.id AS id_unit, u.id_item, u.takentime, u.place, geocoding.level_4 AS level, geocoding.admin_4 AS admin
FROM public.unit u
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_4_person_timeline_view
AS
SELECT
p.id_unit, u.id_item, u.takentime, u.place, geocoding.level_4 AS level, geocoding.admin_4 AS admin, p.id_person
FROM (
SELECT id_unit, array_agg(DISTINCT id_person) AS id_person
FROM public.face GROUP BY id_unit
) p
LEFT JOIN public.unit u ON p.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_4_concept_timeline_view
AS
SELECT
c.id_unit, u.id_item, u.takentime, u.place, geocoding.level_4 AS level, geocoding.admin_4 AS admin, c.id_concept
FROM (
SELECT id_unit, array_agg(DISTINCT id_concept) AS id_concept
FROM public.many_unit_has_many_concept GROUP BY id_unit
) c
LEFT JOIN public.unit u ON c.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_4_general_tag_timeline_view
AS
SELECT
g.id_unit, u.id_item, u.takentime, u.place, geocoding.level_4 AS level, geocoding.admin_4 AS admin, g.id_general_tag
FROM (
SELECT id_unit, array_agg(DISTINCT id_general_tag) AS id_general_tag
FROM public.many_unit_has_many_general_tag GROUP BY id_unit
) g
LEFT JOIN public.unit u ON g.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_5_timeline_view
AS
SELECT
u.id AS id_unit, u.id_item, u.takentime, u.place, geocoding.level_5 AS level, geocoding.admin_5 AS admin
FROM public.unit u
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_5_person_timeline_view
AS
SELECT
p.id_unit, u.id_item, u.takentime, u.place, geocoding.level_5 AS level, geocoding.admin_5 AS admin, p.id_person
FROM (
SELECT id_unit, array_agg(DISTINCT id_person) AS id_person
FROM public.face GROUP BY id_unit
) p
LEFT JOIN public.unit u ON p.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_5_concept_timeline_view
AS
SELECT
c.id_unit, u.id_item, u.takentime, u.place, geocoding.level_5 AS level, geocoding.admin_5 AS admin, c.id_concept
FROM (
SELECT id_unit, array_agg(DISTINCT id_concept) AS id_concept
FROM public.many_unit_has_many_concept GROUP BY id_unit
) c
LEFT JOIN public.unit u ON c.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_5_general_tag_timeline_view
AS
SELECT
g.id_unit, u.id_item, u.takentime, u.place, geocoding.level_5 AS level, geocoding.admin_5 AS admin, g.id_general_tag
FROM (
SELECT id_unit, array_agg(DISTINCT id_general_tag) AS id_general_tag
FROM public.many_unit_has_many_general_tag GROUP BY id_unit
) g
LEFT JOIN public.unit u ON g.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_6_timeline_view
AS
SELECT
u.id AS id_unit, u.id_item, u.takentime, u.place, geocoding.level_6 AS level, geocoding.admin_6 AS admin
FROM public.unit u
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_6_person_timeline_view
AS
SELECT
p.id_unit, u.id_item, u.takentime, u.place, geocoding.level_6 AS level, geocoding.admin_6 AS admin, p.id_person
FROM (
SELECT id_unit, array_agg(DISTINCT id_person) AS id_person
FROM public.face GROUP BY id_unit
) p
LEFT JOIN public.unit u ON p.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_6_concept_timeline_view
AS
SELECT
c.id_unit, u.id_item, u.takentime, u.place, geocoding.level_6 AS level, geocoding.admin_6 AS admin, c.id_concept
FROM (
SELECT id_unit, array_agg(DISTINCT id_concept) AS id_concept
FROM public.many_unit_has_many_concept GROUP BY id_unit
) c
LEFT JOIN public.unit u ON c.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE VIEW public.level_6_general_tag_timeline_view
AS
SELECT
g.id_unit, u.id_item, u.takentime, u.place, geocoding.level_6 AS level, geocoding.admin_6 AS admin, g.id_general_tag
FROM (
SELECT id_unit, array_agg(DISTINCT id_general_tag) AS id_general_tag
FROM public.many_unit_has_many_general_tag GROUP BY id_unit
) g
LEFT JOIN public.unit u ON g.id_unit = u.id
LEFT JOIN public.geocoding ON u.id_geocoding = geocoding.id
WHERE u.id_geocoding IS NOT NULL;
CREATE FUNCTION public.delete_face_trigger_func ()
    RETURNS trigger
    LANGUAGE plpgsql
    VOLATILE
    CALLED ON NULL INPUT
    SECURITY INVOKER
    COST 1
    AS $$
DECLARE
    person_id integer;
    group_id integer;
    cluster_id integer;
    group_weight integer;
    remain_group_count integer;
    remain_cluster_count integer;
    person_group_table text;
    cluster_table text;
    person_table text;
    BEGIN
        person_group_table = TG_TABLE_SCHEMA || '.person_group';
        person_table = TG_TABLE_SCHEMA || '.person';
        cluster_table = TG_TABLE_SCHEMA || '.cluster';
        person_id = OLD.id_person;
        group_id = OLD.id_person_group;
        IF (group_id IS NOT NULL) THEN
            EXECUTE 'SELECT weight, id_cluster FROM '|| person_group_table ||
            ' WHERE id = '|| group_id INTO group_weight, cluster_id;
            if (group_weight > 1) THEN
                EXECUTE 'UPDATE ' || person_group_table || ' SET weight = weight - 1 WHERE id = ' || group_id;
                RETURN NULL;
            END IF;
            EXECUTE 'DELETE FROM ' || person_group_table || ' WHERE id = ' || group_id;
            EXECUTE 'SELECT COUNT(*) FROM ' || person_group_table || ' WHERE id_cluster = ' || cluster_id INTO remain_group_count;
            IF (remain_group_count > 0) THEN
                RETURN NULL;
            END IF;
            EXECUTE 'DELETE FROM '|| cluster_table || ' WHERE id = ' || cluster_id;
        END IF;
        IF (person_id IS NOT NULL) THEN
            EXECUTE 'SELECT COUNT(*) FROM ' || cluster_table || ' WHERE id_person = ' || person_id INTO remain_cluster_count;
            IF (remain_cluster_count > 0) THEN
                RETURN NULL;
            END IF;
            EXECUTE 'DELETE FROM ' || person_table || ' WHERE id = ' || person_id;
        END IF;
        RETURN NULL;
    END;
$$;
CREATE TRIGGER delete_face_trigger
	AFTER DELETE
	ON public.face
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_face_trigger_func();
CREATE VIEW public.folder_timeline_view
AS
SELECT ss.id_item,
  item.type,
  ss.id_unit,
  ss.id_folder,
  ss.takentime,
  ss.place
FROM (
  SELECT
    u.id_item,
    array_agg(DISTINCT u.id) AS id_unit,
    array_agg(DISTINCT u.id_folder) AS id_folder,
    min(u.takentime) AS takentime,
	u.place
  FROM
    folder f, unit u
  WHERE f.id = u.id_folder
  GROUP BY u.id_item, u.place) SS, item
WHERE item.id = SS.id_item;
INSERT INTO public.folder(id, name, parent) VALUES(0, '/', 0);
INSERT INTO public.folder(name, parent) VALUES('/Moments', 0);
