ALTER TABLE public.share ADD COLUMN id_user_info integer DEFAULT NULL;
ALTER TABLE public.share ADD CONSTRAINT share_id_user_info_fk FOREIGN KEY (id_user_info)
REFERENCES public.user_info (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE OR REPLACE VIEW public.share_user_info_group_info_normal_album_view
AS
SELECT
 N.id,
 N.name,
 N.type,
 N.shared,
 N.create_time,
 N.cover,
 N.sort_type,
 N.passphrase_share,
 N.item_count,
 N.start_time,
 N.end_time,
 N.version,
 M.id_group_info,
 M.id_user_info,
 M.create_time AS share_time,
 S.access_permission,
 S.id_user_info AS owner_id_user_info
FROM normal_album as N,
 (SELECT
  T.passphrase_share,
  ARRAY_AGG(T.id_group_info) AS id_group_info,
  ARRAY_AGG(T.id_user_info) AS id_user_info,
  MAX(create_time) AS create_time
  FROM
   (SELECT
    passphrase_share,
    id_group_info,
    create_time,
    0 AS id_user_info
    FROM many_share_has_many_group_info UNION
    SELECT
    passphrase_share,
    0 AS id_group_info,
    create_time,
    id_user_info
    FROM many_share_has_many_user_info
   ) AS T
  GROUP BY T.passphrase_share
 ) AS M,
 share AS S
WHERE N.passphrase_share = M.passphrase_share AND N.passphrase_share = S.passphrase;
ALTER TABLE public.person ADD COLUMN custom_cover bool NOT NULL DEFAULT false;
CREATE OR REPLACE FUNCTION public.update_person_best_cover(id_person integer, target_table text)
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  cover integer;
  BEGIN
    EXECUTE format('SELECT id FROM %I.face WHERE id_person = %s and id_unit != 0 ORDER BY score DESC LIMIT 1',
      target_table, id_person) INTO cover;
    IF (cover IS NOT NULL) THEN
      EXECUTE format('UPDATE %I.person SET cover = %s WHERE id = %s', target_table, cover, id_person);
    END IF;
  END;
$$;
CREATE OR REPLACE FUNCTION public.update_face_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  custom_cover bool;
  cover integer;
  BEGIN
    IF (OLD.id_person != NEW.id_person) THEN
      EXECUTE format('SELECT custom_cover FROM %I.person WHERE id = %s', TG_TABLE_SCHEMA, NEW.id_person) INTO custom_cover;
      IF (custom_cover = false) THEN
        PERFORM public.update_person_best_cover(NEW.id_person, TG_TABLE_SCHEMA);
      END IF;
    END IF;
    IF (OLD.id_person > 0) THEN
      IF (OLD.id_person != NEW.id_person) or (OLD.id_unit != NEW.id_unit) THEN
        EXECUTE format('SELECT cover FROM %I.person WHERE id = %s', TG_TABLE_SCHEMA, OLD.id_person) INTO cover;
        IF (cover = OLD.id) THEN
          PERFORM public.update_person_best_cover(OLD.id_person, TG_TABLE_SCHEMA);
        END IF;
      END IF;
    END IF;
    RETURN NULL;
  END;
$$;
CREATE OR REPLACE FUNCTION public.delete_face_trigger_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  face_id integer;
  person_id integer;
  group_id integer;
  cluster_id integer;
  group_weight integer;
  remain_group_count integer;
  remain_cluster_count integer;
  person_group_table text;
  cluster_table text;
  person_table text;
  cover integer;
  BEGIN
    person_group_table = TG_TABLE_SCHEMA || '.person_group';
    person_table = TG_TABLE_SCHEMA || '.person';
    cluster_table = TG_TABLE_SCHEMA || '.cluster';
    person_id = OLD.id_person;
    group_id = OLD.id_person_group;
    face_id = OLD.id;
    IF (person_id IS NOT NULL) THEN
      EXECUTE format('SELECT cover FROM %I.person WHERE id = %s', TG_TABLE_SCHEMA, person_id) INTO cover;
      IF (cover = OLD.id) THEN
        PERFORM public.update_person_best_cover(person_id, TG_TABLE_SCHEMA);
      END IF;
    END IF;
    IF (group_id IS NOT NULL) THEN
      EXECUTE 'SELECT weight, id_cluster FROM '|| person_group_table ||
      ' WHERE id = '|| group_id INTO group_weight, cluster_id;
      if (group_weight > 1) THEN
        EXECUTE 'UPDATE ' || person_group_table || ' SET weight = weight - 1 WHERE id = ' || group_id;
        RETURN NULL;
      END IF;
      EXECUTE 'DELETE FROM ' || person_group_table || ' WHERE id = ' || group_id;
      EXECUTE 'SELECT COUNT(*) FROM ' || person_group_table || ' WHERE id_cluster = ' || cluster_id INTO remain_group_count;
      IF (remain_group_count > 0) THEN
        RETURN NULL;
      END IF;
      EXECUTE 'DELETE FROM '|| cluster_table || ' WHERE id = ' || cluster_id;
    END IF;
    IF (person_id IS NOT NULL) THEN
      EXECUTE 'SELECT COUNT(*) FROM ' || cluster_table || ' WHERE id_person = ' || person_id INTO remain_cluster_count;
      IF (remain_cluster_count > 0) THEN
        RETURN NULL;
      END IF;
      EXECUTE 'DELETE FROM ' || person_table || ' WHERE id = ' || person_id;
    END IF;
    RETURN NULL;
  END;
$$;
CREATE TRIGGER update_face_trigger
  AFTER UPDATE
  ON public.face
  FOR EACH ROW
  EXECUTE PROCEDURE public.update_face_func();