DROP FUNCTION IF EXISTS share_check_album_func() CASCADE;
CREATE OR REPLACE FUNCTION public.delete_share_cascade_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
  DECLARE
    key text;
    accessed_share_album text;
  BEGIN
    accessed_share_album = TG_TABLE_SCHEMA || '.accessed_share_album';
    key = OLD.passphrase;
    EXECUTE 'DELETE FROM ' || accessed_share_album || ' WHERE passphrase = ' || quote_literal(key);
    RETURN NULL;
  END
$$;
DROP FUNCTION IF EXISTS public.generate_user_schema() CASCADE;
CREATE OR REPLACE FUNCTION public.generate_user_schema ( version integer)
  RETURNS text
  LANGUAGE c
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY DEFINER
  COST 1
  AS '/var/packages/SynologyMoments/target/usr/lib/libsynophoto-db-trigger.so', 'generate_user_schema';
CREATE OR REPLACE FUNCTION public.update_user_schema( old_version integer, new_version integer)
	RETURNS text
	LANGUAGE c
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY DEFINER
	COST 1
	AS '/var/packages/SynologyMoments/target/usr/lib/libsynophoto-db-trigger.so', 'update_user_schema';
CREATE OR REPLACE FUNCTION public.create_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
schema_name     text;
schema_sql      text;
t               text;
version         integer;
inherit_table   text[] = '{
    "unit", "video_additional", "live_additional", "burst_additional",
    "item",
    "album", "condition_album", "normal_album",
    "face", "person", "cluster",  "person_group",
    "folder", "general_tag",
    "geocoding", "geocoding_info",  "address",
    "many_share_has_many_group_info",
    "many_share_has_many_user_info",
    "many_unit_has_many_concept",
    "many_unit_has_many_general_tag",
    "many_item_has_many_normal_album",
    "metadata",
    "share", "share_comment",
    "thumbnail",
    "version_time",
    "delete_item", "delete_album",
    "notification",
    "thumb_preview",
    "search_timeline",
    "accessed_share_album",
    "video_convert"}';
BEGIN
    schema_name = 'user_'|| NEW.id;
    EXECUTE format('SELECT value FROM public.config WHERE key=%L', 'version') INTO version;
    EXECUTE format('CREATE SCHEMA %s', schema_name);
    EXECUTE format('SET LOCAL search_path TO %L, %L', schema_name, 'public');
    SELECT public.generate_user_schema(version) INTO schema_sql;
    EXECUTE schema_sql;
    FOREACH t IN ARRAY inherit_table
    LOOP
        EXECUTE format('ALTER TABLE %I INHERIT public.%I', t, t);
    END LOOP;
    EXECUTE format('SET LOCAL search_path TO %L', 'public');
    RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.update_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  schema text;
  schema_sql text;
  version integer;
  old_version integer;
BEGIN
  schema = 'user_' || NEW.id;
  IF (NEW.id != OLD.id) THEN
    EXECUTE format('ALTER SCHEMA user_%s RENAME TO user_%s', OLD.id, NEW.id);
  END IF;
  version = NEW.config->'version';
  old_version = OLD.config->'version';
  SELECT COALESCE(old_version, 1) INTO old_version;
  SELECT COALESCE(version, 1) INTO version;
  IF (old_version != version) THEN
    SELECT update_user_schema(old_version, version) INTO schema_sql;
    IF schema_sql != '' THEN
      EXECUTE format('SET LOCAL search_path TO %L, %L', 'user_' || NEW.id, 'public');
      EXECUTE schema_sql;
      EXECUTE format('SET LOCAL search_path TO %L', 'public');
    END IF;
  END IF;
  RETURN NULL;
END
$$;
CREATE TRIGGER update_user_schema_trigger
	AFTER UPDATE
	ON public.user_info
	FOR EACH ROW
	EXECUTE PROCEDURE public.update_user_schema_func();
ALTER TABLE folder ADD COLUMN name_for_sort text NOT NULL DEFAULT '';
UPDATE folder SET name_for_sort = '/' WHERE id = 0;
UPDATE folder SET name_for_sort = 'MOMENTS' WHERE id = 1;
CREATE TABLE public.video_convert(
	duration int8 NOT NULL,
	quality text NOT NULL DEFAULT '',
	video_info json NOT NULL DEFAULT '{}',
	audio_info json NOT NULL DEFAULT '{}',
	id_unit integer
);
ALTER TABLE public.video_convert ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX video_convert_id_unit_idx ON public.video_convert
	USING btree
	(
	  id_unit ASC NULLS LAST
	);