CREATE OR REPLACE FUNCTION public.get_unit_takentime_by_album(id_album int4, schema text)
  RETURNS TABLE (max bigint, min bigint)
  LANGUAGE plpgsql
  STABLE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  unit_table text;
  many_table text;
BEGIN
    unit_table = schema || '.unit';
    many_table = schema || '.many_item_has_many_normal_album';
    RETURN QUERY EXECUTE 'SELECT MAX(takentime), MIN(takentime) FROM ' || unit_table || ' WHERE id_item IN(
        SELECT id_item FROM '|| many_table || ' WHERE id_normal_album = ' || id_album ||')';
END;
$$;
CREATE OR REPLACE FUNCTION public.update_normal_album_trigger_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
    album_id integer;
    item_id integer;
    is_delete bool;
    album_start_time bigint;
    album_end_time bigint;
    unit_time bigint;
    unit_table text;
    normal_album_table text;
    album_takentime_max bigint;
    album_takentime_min bigint;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
  IF (TG_OP = 'INSERT') THEN
    album_id = NEW.id_normal_album;
    item_id  = NEW.id_item;
    is_delete= false;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) + 1 WHERE id =' || album_id;
  ELSIF (TG_OP = 'DELETE') THEN
    album_id = OLD.id_normal_album;
    item_id  = OLD.id_item;
    is_delete= true;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) - 1 WHERE id =' || album_id;
  END IF;
  EXECUTE 'SELECT takentime FROM ' || unit_table || ' WHERE id_item = ' || item_id INTO unit_time;
  EXECUTE 'SELECT COALESCE(start_time, 0), COALESCE(end_time, 0) FROM ' || normal_album_table ||
          ' WHERE id =' || album_id INTO album_start_time, album_end_time;
  IF (unit_time IS NULL OR (is_delete AND (unit_time = album_start_time OR unit_time = album_end_time))) THEN
    EXECUTE format('SELECT COALESCE(max, 0), COALESCE(min, 0) FROM public.get_unit_takentime_by_album(%L, %L)', album_id, TG_TABLE_SCHEMA) INTO album_takentime_max, album_takentime_min;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || album_takentime_min || ', end_time = ' || album_takentime_max || ' WHERE id = ' || album_id;
  ELSE
    IF (album_start_time = 0 OR unit_time < album_start_time) THEN
      EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || unit_time || ' WHERE id = ' || album_id;
    END IF;
    IF (album_end_time = 0 OR unit_time > album_end_time) THEN
      EXECUTE 'UPDATE ' || normal_album_table || ' SET end_time = ' || unit_time || ' WHERE id = ' || album_id;
    END IF;
  END IF;
  RETURN NULL;
END;
$$;
DROP FUNCTION IF EXISTS public.get_unit_takentime_by_album(int4, text, text);
CREATE OR REPLACE FUNCTION public.update_normal_album_time_trigger_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  id_item integer;
  many_table text;
  normal_album_table text;
  max_time bigint;
  min_time bigint;
  record RECORD;
BEGIN
  id_item  = OLD.id_item;
  normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
  many_table = TG_TABLE_SCHEMA || '.many_item_has_many_normal_album';
  FOR record IN EXECUTE format('SELECT * FROM %s WHERE id_item = %L', many_table, id_item) LOOP
    EXECUTE format('SELECT max, min FROM public.get_unit_takentime_by_album(%L, %L)', record.id_normal_album, TG_TABLE_SCHEMA) INTO max_time, min_time;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || min_time || ', end_time = ' || max_time || ' WHERE id = ' || record.id_normal_album;
  END LOOP;
  RETURN NULL;
END;
$$;
CREATE TRIGGER unit_update_takentime_cascade_to_noraml_album_trigger
  AFTER UPDATE
  OF takentime ON public.unit
  FOR EACH ROW
  EXECUTE PROCEDURE public.update_normal_album_time_trigger_func();