CREATE OR REPLACE VIEW public.person_face_view
AS
SELECT
    face.id,
    id_person,
    id_unit,
    score
FROM public.face, public.unit
WHERE unit.id = face.id_unit AND unit.place <= 0;
CREATE OR REPLACE VIEW public.drive_person_face_view
AS
SELECT
    face.id,
    id_person,
    id_unit,
    score
FROM public.face, public.unit
WHERE unit.id = face.id_unit AND unit.place <= 1;
CREATE FUNCTION public.delete_nonexisting_geocoding_after_delete_unit()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  geocoding_id integer;
  geocoding_table text;
  unit_count integer;
  unit_table text;
BEGIN
  IF (OLD.id_geocoding IS NULL) THEN
    RETURN OLD;
  END IF;
  unit_table = TG_TABLE_SCHEMA || '.unit';
  geocoding_table = TG_TABLE_SCHEMA || '.geocoding';
  geocoding_id = OLD.id_geocoding;
  EXECUTE 'SELECT COUNT(*) FROM ' || unit_table || ' WHERE id_geocoding = ' || geocoding_id INTO unit_count;
  IF (0 = unit_count) THEN
    EXECUTE 'DELETE FROM ' || geocoding_table || ' WHERE id = ' || geocoding_id;
  END IF;
  RETURN OLD;
END;
$$;
CREATE TRIGGER update_geocoding_after_delete_unit_trigger
  AFTER DELETE
  ON public.unit
  FOR EACH ROW
  EXECUTE PROCEDURE public.delete_nonexisting_geocoding_after_delete_unit();