CREATE OR REPLACE FUNCTION public.delete_unit_version_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  count integer;
  new_version bigint;
  unit_table text;
  delete_item_table text;
  id_item integer;
  id_user integer;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  IF (TG_OP = 'DELETE') THEN
    id_item = OLD.id_item;
    EXECUTE format('SELECT count(*) FROM %s WHERE id_item = %L', unit_table, id_item) INTO count;
    SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
    IF (count > 0) THEN
      EXECUTE format('UPDATE %s SET version = %L  WHERE id_item = %L', unit_table, new_version, id_item);
    ELSE
      EXECUTE format('INSERT INTO %s (id_item, version) VALUES (%L, %L)', delete_item_table, id_item, new_version);
    END IF;
    EXECUTE format('SELECT split_part(%L, %L, 2)', TG_TABLE_SCHEMA, '_') into id_user;
    EXECUTE format('DELETE FROM index_queue WHERE id_user = %L and id_unit = %L', id_user, OLD.id);
  ELSIF (TG_OP = 'UPDATE' AND OLD.reindex_flag = NEW.reindex_flag) THEN
    id_item = OLD.id_item;
  END IF;
  IF id_item IS NOT NULL THEN
    EXECUTE format('SELECT public.unit_update_normal_album_func(%L, %L)', TG_TABLE_SCHEMA, id_item);
  END IF;
  RETURN NULL;
END
$$;
CREATE INDEX folder_name_idx ON public.folder
  USING btree
  (
    name text_pattern_ops
  );