CREATE OR REPLACE FUNCTION public.delete_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  count integer;
  new_version bigint;
  unit_table text;
  delete_item_table text;
  id_item integer;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  IF (TG_OP = 'DELETE') THEN
    id_item = OLD.id_item;
    EXECUTE format('SELECT count(*) FROM %s WHERE id_item = %L', unit_table, id_item) INTO count;
    SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
    IF (count > 0) THEN
      EXECUTE format('UPDATE %s SET version = %L  WHERE id_item = %L', unit_table, new_version, id_item);
    ELSE
      EXECUTE format('INSERT INTO %s (id_item, version) VALUES (%L, %L)', delete_item_table, id_item, new_version);
    END IF;
  ELSIF (TG_OP = 'UPDATE') THEN
    id_item = OLD.id_item;
  END IF;
  IF id_item IS NOT NULL THEN
    EXECUTE format('SELECT public.unit_update_normal_album_func(%L, %L)', TG_TABLE_SCHEMA, id_item);
  END IF;
  RETURN NULL;
END
$$;
DROP TRIGGER IF EXISTS update_unit_after_delete_trigger ON public.unit CASCADE;
CREATE TRIGGER update_unit_after_delete_trigger
	AFTER DELETE OR UPDATE
	ON public.unit
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_unit_version_func();
CREATE OR REPLACE FUNCTION public.update_normal_album_cover_func ( schema text,  album_id integer)
	RETURNS void
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    many_item_has_many_normal_album_table text;
    normal_album_table  text;
    item_id integer;
BEGIN
    many_item_has_many_normal_album_table = schema || '.many_item_has_many_normal_album';
    normal_album_table = schema || '.normal_album';
    EXECUTE format('SELECT id_item FROM %s WHERE id_normal_album = %L', many_item_has_many_normal_album_table, album_id) INTO item_id;
    IF item_id IS NULL THEN
      item_id = 0;
    END IF;
    EXECUTE format('UPDATE %s SET cover = %L WHERE id = %L', normal_album_table, item_id, album_id);
END
$$;
CREATE OR REPLACE FUNCTION public.increase_normal_album_version_func ( schema text,  album_id integer)
	RETURNS void
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    normal_album_table  text;
    new_version integer;
    item_id integer;
BEGIN
    normal_album_table = schema || '.normal_album';
    SELECT public.get_next_version_func(schema) INTO new_version;
    EXECUTE format('UPDATE %s SET version = %L WHERE id = %L', normal_album_table, new_version, album_id);
END
$$;
CREATE OR REPLACE FUNCTION public.unit_update_normal_album_func ( schema text,  item_id integer)
	RETURNS void
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  normal_album_table text;
  unit_table text;
  count integer;
  record RECORD;
BEGIN
  normal_album_table = schema || '.normal_album';
  unit_table = schema || '.unit';
  EXECUTE format('SELECT count(*) FROM %s WHERE id_item = %L', unit_table, item_id) INTO count;
  IF (count > 0) THEN
    FOR record IN EXECUTE format('SELECT * FROM %s WHERE cover = %L', normal_album_table, item_id) LOOP
      EXECUTE format('SELECT public.increase_normal_album_version_func(%L, %s)', schema, record.id);
    END LOOP;
  ELSE
    FOR record IN EXECUTE format('SELECT * FROM %s WHERE cover = %L', normal_album_table, item_id) LOOP
      EXECUTE format('SELECT public.update_normal_album_cover_func(%L, %s)', schema, record.id);
    END LOOP;
  END IF;
END
$$;
CREATE OR REPLACE VIEW public.recently_added_timeline_view
AS
SELECT
 item.id as id_item,
 item.type,
 min(unit.createtime)/1000 as createtime,
 array_agg(unit.id) as id_unit,
 unit.place
FROM item , unit
WHERE unit.id_item = item.id AND createtime > (extract(epoch from now())-2592000)
GROUP BY item.id, unit.place
;