CREATE TABLE public.assessment(
  id_unit integer,
  score integer NOT NULL,
  standard_deviation integer NOT NULL DEFAULT 0,
  CONSTRAINT assessment_id_unit_pk PRIMARY KEY (id_unit)
);
ALTER TABLE public.assessment ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX assessment_score_idx ON public.assessment
  USING btree
  (
    score DESC NULLS LAST
  );
CREATE INDEX assessment_standard_deviation_idx ON public.assessment
  USING btree
  (
    standard_deviation ASC NULLS LAST
  );
CREATE TABLE public.enhancement(
  id_unit integer,
  type int2 NOT NULL,
  param json NOT NULL,
  applied bool NOT NULL DEFAULT false,
  thumb_ready bool NOT NULL DEFAULT false,
  CONSTRAINT enhancement_id_unit_type_pk PRIMARY KEY (id_unit,type)
);
ALTER TABLE public.enhancement ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX enhancement_id_unit_idx ON public.enhancement
  USING btree
  (
    id_unit ASC NULLS LAST
  );
CREATE INDEX enhancement_type_idx ON public.enhancement
  USING btree
  (
    type ASC NULLS LAST
  );
ALTER TABLE public.live_additional ADD COLUMN type int2 NOT NULL;
CREATE INDEX live_additional_grouping_key_idx ON public.live_additional
	USING btree
	(
	  grouping_key ASC NULLS LAST
	);
CREATE INDEX burst_additional_grouping_key_idx ON public.burst_additional
	USING btree
	(
	  grouping_key ASC NULLS LAST
	);
CREATE FUNCTION public.update_burst_additional_sequence_func()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  current_sequence smallint;
  count smallint;
  burst_additional text;
BEGIN
  burst_additional = TG_TABLE_SCHEMA || '.burst_additional';
  EXECUTE format('SELECT count(*) FROM %s WHERE grouping_key = %L', burst_additional, NEW.grouping_key) INTO count;
  IF (count > 0) THEN
    EXECUTE format('SELECT sequence FROM %s WHERE grouping_key = %L ORDER BY sequence DESC limit 1', burst_additional, NEW.grouping_key) INTO current_sequence;
    NEW.sequence = current_sequence + 1;
  ELSE
    NEW.sequence = 1;
  END IF;
  RETURN NEW;
END
$$;
CREATE TRIGGER update_burst_additional_sequence_trigger
	BEFORE INSERT
	ON public.burst_additional
	FOR EACH ROW
	EXECUTE PROCEDURE public.update_burst_additional_sequence_func();
ALTER TABLE public.delete_album DROP CONSTRAINT IF EXISTS version_fk CASCADE;
ALTER TABLE public.delete_album ADD CONSTRAINT version_fk FOREIGN KEY (version)
REFERENCES public.version_time (version) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.delete_item DROP CONSTRAINT IF EXISTS version_fk CASCADE;
ALTER TABLE public.delete_item ADD CONSTRAINT version_fk FOREIGN KEY (version)
REFERENCES public.version_time (version) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE OR REPLACE FUNCTION public.delete_expired_version_time_data_func()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  count bigint;
  version_time text;
BEGIN
  version_time = TG_TABLE_SCHEMA || '.version_time';
  EXECUTE format('SELECT count(*) FROM %s', version_time) INTO count;
  IF (count > 110000) THEN
		EXECUTE format('DELETE FROM %s WHERE version IN (SELECT version FROM %s order by version limit %L)', version_time, version_time, 10000);
  END IF;
	RETURN NEW;
END
$$;
DROP TRIGGER IF EXISTS delete_expired_version_time_data_trigger ON public.version_time CASCADE;
CREATE TRIGGER delete_expired_version_time_data_trigger
	BEFORE INSERT
	ON public.version_time
	FOR EACH ROW
	EXECUTE PROCEDURE public.delete_expired_version_time_data_func();
ALTER TABLE unit ADD COLUMN enhancement_applied bool NOT NULL DEFAULT false;
CREATE OR REPLACE FUNCTION public.update_unit_version_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
    new_version bigint;
BEGIN
    IF (TG_OP = 'INSERT') THEN
        SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
        NEW.version = new_version;
        RETURN NEW;
    ELSIF (TG_OP = 'UPDATE') THEN
        IF (NEW.index_stage != OLD.index_stage OR
            NEW.filename != OLD.filename OR
            NEW.id_folder != OLD.id_folder OR
            NEW.place != OLD.place OR
            NEW.takentime != OLD.takentime OR
            NEW.id_geocoding != OLD.id_geocoding OR
            NEW.mtime != OLD.mtime) THEN
           SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
           NEW.version = new_version;
        END IF;
        RETURN NEW;
    END IF;
    RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.delete_unit_version_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  count integer;
  new_version bigint;
  unit_table text;
  delete_item_table text;
  id_item integer;
  id_user integer;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  IF (TG_OP = 'DELETE') THEN
    id_item = OLD.id_item;
    EXECUTE format('SELECT count(*) FROM %s WHERE id_item = %L', unit_table, id_item) INTO count;
    SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
    IF (count > 0) THEN
      EXECUTE format('UPDATE %s SET version = %L  WHERE id_item = %L', unit_table, new_version, id_item);
    END IF;
    EXECUTE format('SELECT split_part(%L, %L, 2)', TG_TABLE_SCHEMA, '_') into id_user;
    EXECUTE format('DELETE FROM index_queue WHERE id_user = %L and id_unit = %L', id_user, OLD.id);
  ELSIF (TG_OP = 'UPDATE' AND OLD.reindex_flag = NEW.reindex_flag) THEN
    id_item = OLD.id_item;
  END IF;
  IF id_item IS NOT NULL THEN
    EXECUTE format('SELECT public.unit_update_normal_album_func(%L, %L)', TG_TABLE_SCHEMA, id_item);
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.delete_item_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  delete_item_table text;
  new_version bigint;
BEGIN
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
  EXECUTE format('INSERT INTO %s (id_item, version) VALUES (%L, %L)', delete_item_table, OLD.id, new_version);
  RETURN NULL;
END
$$;
DROP TRIGGER IF EXISTS delete_item_trigger ON public.item CASCADE;
CREATE TRIGGER delete_item_trigger
  AFTER DELETE
  ON public.item
  FOR EACH ROW
  EXECUTE PROCEDURE public.delete_item_func();
CREATE TABLE public.discover_style(
  id_unit integer,
  score integer NOT NULL,
  discover_group integer NOT NULL,
  recommand bool NOT NULL DEFAULT false,
  applied bool NOT NULL DEFAULT false,
  hidden bool NOT NULL DEFAULT false,
  CONSTRAINT discover_style_id_unit_pk PRIMARY KEY (id_unit)
);
ALTER TABLE public.discover_style ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX discover_style_score_idx ON public.discover_style
  USING btree
  (
    score DESC NULLS LAST
  );
CREATE INDEX discover_style_recommand_idx ON public.discover_style
  USING btree
  (
    recommand DESC NULLS LAST
  );
CREATE INDEX discover_style_discover_group_idx ON public.discover_style
  USING btree
  (
    discover_group DESC NULLS LAST
  );
CREATE VIEW public.discover_style_unit_view
AS
SELECT
  D.id_unit,
  U.id_item,
  D.score,
  D.discover_group,
  D.applied,
  U.place
FROM
 public.discover_style as D, unit as U
WHERE
 D.id_unit = U.id AND D.recommand = true AND D.hidden = false;
CREATE TABLE public.similar_group(
  id serial NOT NULL,
  cover integer not NULL,
  item_count int NOT NULL DEFAULT 0,
  item_count_drive int NOT NULL DEFAULT 0,
  CONSTRAINT similar_group_pk PRIMARY KEY (id)
);
CREATE TABLE public.many_similar_group_has_many_item(
  id_item integer,
  id_similar_group integer,
  hidden bool NOT NULL DEFAULT false
);
ALTER TABLE public.many_similar_group_has_many_item ADD CONSTRAINT group_fk FOREIGN KEY (id_similar_group)
REFERENCES public.similar_group (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE public.many_similar_group_has_many_item ADD CONSTRAINT item_fk FOREIGN KEY (id_item)
REFERENCES public.item (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE INDEX many_similar_group_has_many_item_idx ON public.many_similar_group_has_many_item
USING btree
(
  id_similar_group ASC NULLS LAST,
  hidden ASC NULLS LAST
);
CREATE TABLE public.unit_similar_hash(
  id_unit integer,
  perceptual_hash text NOT NULL DEFAULT '',
  CONSTRAINT unit_extra_info_pk PRIMARY KEY (id_unit)
);
ALTER TABLE public.unit_similar_hash ADD CONSTRAINT unit_fk FOREIGN KEY (id_unit)
REFERENCES public.unit (id) MATCH FULL
ON DELETE CASCADE ON UPDATE CASCADE;
CREATE OR REPLACE FUNCTION public.update_similar_info_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  cover text;
  current_cover bigint;
  item_count bigint;
  visable_count bigint;
  similar_group_id bigint;
  need_update_cover bool;
BEGIN
  similar_group_table = TG_TABLE_SCHEMA || '.similar_group';
  many_similar_group_has_many_item_table = TG_TABLE_SCHEMA || '.many_similar_group_has_many_item';
  need_update_cover = false;
  IF (TG_OP = 'DELETE') THEN
    similar_group_id = OLD.id_similar_group;
  ELSE
    similar_group_id = NEW.id_similar_group;
  END IF;
  EXECUTE format('SELECT count(*) FROM %s WHERE id_similar_group = %L GROUP BY id_similar_group', many_similar_group_has_many_item_table, similar_group_id) INTO item_count;
  EXECUTE format('SELECT cover FROM %s WHERE id = %L', similar_group_table, similar_group_id) INTO current_cover;
  IF (TG_OP = 'INSERT' AND current_cover IS NULL) THEN
    need_update_cover = true;
  ELSIF (TG_OP = 'UPDATE' AND current_cover = NEW.id_item AND NEW.hidden = 't') THEN
    need_update_cover = true;
  ELSIF (TG_OP = 'DELETE' AND current_cover = OLD.id_item) THEN
    need_update_cover = true;
  END IF;
  IF (item_count IS NULL) THEN
    EXECUTE format('DELETE FROM %s WHERE id = %L', similar_group_table, similar_group_id);
    RETURN NULL;
  END IF;
  IF (need_update_cover OR current_cover = 0) THEN
    EXECUTE format('SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item limit 1', many_similar_group_has_many_item_table, similar_group_id) INTO cover;
    IF (cover IS NULL) THEN
      cover = 0;
    END IF;
  ELSE
    cover = current_cover;
  END IF;
  EXECUTE format('SELECT count(*) FROM %s WHERE id_similar_group = %L AND hidden = false GROUP BY id_similar_group', many_similar_group_has_many_item_table, similar_group_id) INTO visable_count;
  IF (visable_count IS NULL) THEN
    visable_count = 0;
  END IF;
  EXECUTE format('UPDATE %s SET cover = %L, item_count = %L WHERE id = %L', similar_group_table, cover, visable_count, similar_group_id);
  RETURN NULL;
END
$$;
CREATE TRIGGER update_similar_group_trigger
  AFTER INSERT OR UPDATE OR DELETE
  ON public.many_similar_group_has_many_item
  FOR EACH ROW
  EXECUTE PROCEDURE public.update_similar_info_func();
CREATE OR REPLACE VIEW public.discover_style_unit_view
AS
SELECT
  D.id_unit,
  U.id_item,
  D.score,
  D.discover_group,
  D.applied,
  U.place
FROM
 public.discover_style as D, unit as U
WHERE
 D.id_unit = U.id AND D.recommand = true AND D.hidden = false
ORDER BY
 D.discover_group DESC, D.score DESC, D.id_unit;
CREATE OR REPLACE VIEW public.assessment_unit_view
AS
SELECT
  A.id_unit,
  A.score,
  U.type,
  U.place,
  A.standard_deviation
FROM
 public.assessment as A, unit as U
WHERE
 A.id_unit = U.id;
CREATE OR REPLACE FUNCTION public.create_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
schema_name     text;
schema_sql      text;
t               text;
version         integer;
inherit_table   text[] = '{
    "unit", "video_additional", "live_additional", "burst_additional",
    "item",
    "album", "condition_album", "normal_album",
    "face", "person", "cluster",  "person_group",
    "folder", "general_tag",
    "geocoding", "geocoding_info",  "address",
    "many_share_has_many_group_info",
    "many_share_has_many_user_info",
    "many_unit_has_many_concept",
    "many_unit_has_many_concept_hidden",
    "many_unit_has_many_general_tag",
    "many_item_has_many_normal_album",
    "metadata",
    "share", "share_comment",
    "thumbnail",
    "version_time",
    "delete_item", "delete_album",
    "notification",
    "thumb_preview",
    "search_timeline",
    "accessed_share_album",
    "video_convert", "assessment", "enhancement", "discover_style",
    "many_similar_group_has_many_item", "unit_similar_hash", "similar_group"}';
BEGIN
    schema_name = 'user_'|| NEW.id;
    EXECUTE format('SELECT value FROM public.config WHERE key=%L', 'version') INTO version;
    EXECUTE format('CREATE SCHEMA %s', schema_name);
    EXECUTE format('SET LOCAL search_path TO %L, %L', schema_name, 'public');
    SELECT public.generate_user_schema(version) INTO schema_sql;
    EXECUTE schema_sql;
    FOREACH t IN ARRAY inherit_table
    LOOP
        EXECUTE format('ALTER TABLE %I INHERIT public.%I', t, t);
    END LOOP;
    EXECUTE format('SET LOCAL search_path TO %L', 'public');
    RETURN NULL;
END;
$$;