CREATE OR REPLACE FUNCTION public.create_delete_team_user_trigger_func ()
    RETURNS trigger
    LANGUAGE plpgsql
    VOLATILE
    CALLED ON NULL INPUT
    SECURITY INVOKER
    COST 1
    AS $$
DECLARE
  version integer;
  name text;
  id integer;
BEGIN
  EXECUTE format('SELECT value FROM config WHERE key = %L', 'version') INTO version;
  IF (TG_OP = 'INSERT') THEN
    EXECUTE format('INSERT INTO user_info (name, config, enable) VALUES (%L, \'{"version"%s %s}\', true) RETURNING id', NEW.path, ':', version) INTO id;
    EXECUTE format('DELETE FROM user_%s.folder WHERE id = 1', id);
    EXECUTE format('INSERT INTO user_flag (id_user_info, flag, value) VALUES (%L, \'update_file_mtime_done\', \'true\')', id);
  ELSEIF (TG_OP = 'UPDATE') THEN
    EXECUTE format('UPDATE user_info SET name = %L WHERE name = %L', NEW.path, OLD.path);
  ELSEIF (TG_OP = 'DELETE') THEN
    EXECUTE format('DELETE FROM user_info WHERE name = %L AND uid IS NULL', OLD.path);
  END IF;
  RETURN NULL;
END
$$;
DROP TRIGGER create_delete_team_user_trigger ON team_library;
CREATE TRIGGER create_delete_team_user_trigger
	AFTER INSERT OR UPDATE OR DELETE
	ON public.team_library
	FOR EACH ROW
	EXECUTE PROCEDURE public.create_delete_team_user_trigger_func();
CREATE OR REPLACE VIEW public.concept_album_view AS
 SELECT ss.item_count,
    c.stem,
    ss.id_concept,
    c.threshold,
    c.parent,
    c.sort_index
   FROM public.concept c,
    ( SELECT count(DISTINCT(u.id_item)) AS item_count,
            r.id_concept
           FROM public.many_unit_has_many_concept r,
            public.unit u
          WHERE ((u.id = r.id_unit) AND (u.place <= 0))
          GROUP BY r.id_concept) ss
  WHERE ((c.id = ss.id_concept) AND (ss.item_count >= c.threshold));
CREATE OR REPLACE VIEW public.drive_concept_album_view AS
 SELECT ss.item_count,
    c.stem,
    ss.id_concept,
    c.threshold,
    c.parent,
    c.sort_index
   FROM public.concept c,
    ( SELECT count(DISTINCT(u.id_item)) AS item_count,
            r.id_concept
           FROM public.many_unit_has_many_concept r,
            public.unit u
          WHERE ((u.id = r.id_unit) AND (u.place <= 1))
          GROUP BY r.id_concept) ss
  WHERE ((c.id = ss.id_concept) AND (ss.item_count >= c.threshold));
CREATE OR REPLACE VIEW public.general_tag_album_view AS
 SELECT t.id AS id_general_tag,
    t.name,
    ss.item_count
   FROM public.general_tag t,
    ( SELECT count(DISTINCT(u.id_item)) AS item_count,
            r.id_general_tag
           FROM public.many_unit_has_many_general_tag r,
            public.unit u
          WHERE ((u.id = r.id_unit) AND (u.place <= 0))
          GROUP BY r.id_general_tag) ss
  WHERE (t.id = ss.id_general_tag);
CREATE OR REPLACE VIEW public.drive_general_tag_album_view AS
 SELECT t.id AS id_general_tag,
    t.name,
    ss.item_count
   FROM public.general_tag t,
    ( SELECT count(DISTINCT(u.id_item)) AS item_count,
            r.id_general_tag
           FROM public.many_unit_has_many_general_tag r,
            public.unit u
          WHERE ((u.id = r.id_unit) AND (u.place <= 1))
          GROUP BY r.id_general_tag) ss
  WHERE (t.id = ss.id_general_tag);
CREATE OR REPLACE VIEW public.geocoding_album_view AS
 SELECT s.id_geocoding,
    s.item_count,
    album_count_table.count AS album_count,
    geocoding.level_1
   FROM public.geocoding,
    ( SELECT u.id_geocoding,
            count(DISTINCT(u.id_item)) AS item_count
           FROM public.unit u,
            public.item i
          WHERE ((u.id_item = i.id) AND (u.place <= 0))
          GROUP BY u.id_geocoding
         HAVING (u.id_geocoding IS NOT NULL)) s,
    ( SELECT geocoding_1.level_1,
            count(*) AS count
           FROM public.geocoding geocoding_1
          GROUP BY geocoding_1.level_1) album_count_table
  WHERE ((geocoding.id = s.id_geocoding) AND (geocoding.level_1 = album_count_table.level_1));
CREATE OR REPLACE VIEW public.drive_geocoding_album_view AS
 SELECT s.id_geocoding,
    s.item_count,
    album_count_table.count AS album_count,
    geocoding.level_1
   FROM public.geocoding,
    ( SELECT u.id_geocoding,
            count(DISTINCT(u.id_item)) AS item_count
           FROM public.unit u,
            public.item i
          WHERE ((u.id_item = i.id) AND (u.place <= 1))
          GROUP BY u.id_geocoding
         HAVING (u.id_geocoding IS NOT NULL)) s,
    ( SELECT geocoding_1.level_1,
            count(*) AS count
           FROM public.geocoding geocoding_1
          GROUP BY geocoding_1.level_1) album_count_table
  WHERE ((geocoding.id = s.id_geocoding) AND (geocoding.level_1 = album_count_table.level_1));
ALTER TABLE public.unit RENAME COLUMN content_hash TO cache_key;
CREATE OR REPLACE FUNCTION public.update_unit_version_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
    new_version bigint;
BEGIN
    IF (TG_OP = 'INSERT') THEN
        SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
        NEW.version = new_version;
        RETURN NEW;
    ELSIF (TG_OP = 'UPDATE') THEN
        IF (NEW.index_stage != OLD.index_stage OR
            NEW.filename != OLD.filename OR
            NEW.id_folder != OLD.id_folder OR
            NEW.place != OLD.place OR
            NEW.takentime != OLD.takentime OR
            NEW.id_geocoding != OLD.id_geocoding OR
            NEW.mtime != OLD.mtime OR
            NEW.cache_key != OLD.cache_key) THEN
           SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
           NEW.version = new_version;
        END IF;
        RETURN NEW;
    END IF;
    RETURN NULL;
END
$$;
DROP VIEW IF EXISTS public.live;
CREATE OR REPLACE VIEW public.live
AS
SELECT
   live_unit.id,
   live_unit.type,
   live_unit.filename,
   live_unit.filesize,
   live_unit.createtime,
   live_unit.takentime,
   live_unit.duplicate_hash,
   live_unit.cache_key,
   live_unit.resolution,
   live_unit.index_stage,
   live_unit.id_drive,
   live_unit.place,
   live_unit.version,
   live_unit.id_item,
   live_unit.id_folder,
   live_unit.id_geocoding,
   live_additional.grouping_key,
   live_additional.type AS live_type
FROM
   live_additional,
   (SELECT * FROM unit WHERE type = 3) AS live_unit
WHERE
   live_unit.id = live_additional.id_unit;
DROP VIEW public.burst;
CREATE VIEW public.burst
AS
SELECT
   burst_unit.id,
   burst_unit.type,
   burst_unit.filename,
   burst_unit.filesize,
   burst_unit.createtime,
   burst_unit.takentime,
   burst_unit.duplicate_hash,
   burst_unit.cache_key,
   burst_unit.resolution,
   burst_unit.index_stage,
   burst_unit.id_drive,
   burst_unit.place,
   burst_unit.version,
   burst_unit.id_item,
   burst_unit.id_folder,
   burst_unit.id_geocoding,
   burst_additional.grouping_key,
   burst_additional.sequence,
   burst_additional.major
FROM
   burst_additional,
   (SELECT * FROM unit WHERE type = 2) AS burst_unit
WHERE
   burst_unit.id = burst_additional.id_unit;
DROP VIEW public.video;
CREATE VIEW public.video
AS
SELECT
   video_unit.id,
   video_unit.type,
   video_unit.filename,
   video_unit.filesize,
   video_unit.createtime,
   video_unit.takentime,
   video_unit.duplicate_hash,
   video_unit.cache_key,
   video_unit.resolution,
   video_unit.index_stage,
   video_unit.id_drive,
   video_unit.place,
   video_unit.version,
   video_unit.id_item,
   video_unit.id_folder,
   video_unit.id_geocoding,
   video_additional.duration,
   video_additional.video_info,
   video_additional.audio_info
FROM
   video_additional,
   (SELECT * FROM unit WHERE type = 1) AS video_unit
WHERE
   video_unit.id = video_additional.id_unit;