CREATE OR REPLACE FUNCTION public.update_similar_group_count(schema text, similar_group_id integer)
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  visable_count bigint;
  visable_count_drive bigint;
BEGIN
  similar_group_table = schema || '.similar_group';
  many_similar_group_has_many_item_table = schema || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT COUNT(DISTINCT(id_item)) from unit WHERE id_item in (SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item) AND place <= 0', many_similar_group_has_many_item_table, similar_group_id) INTO visable_count;
  IF (visable_count IS NULL) THEN
    visable_count = 0;
  END IF;
  EXECUTE format('SELECT COUNT(DISTINCT(id_item)) from unit WHERE id_item in (SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item) AND place <= 1', many_similar_group_has_many_item_table, similar_group_id) INTO visable_count_drive;
  IF (visable_count_drive IS NULL) THEN
    visable_count_drive = 0;
  END IF;
  EXECUTE format('UPDATE %s SET item_count = %L, item_count_drive = %L WHERE id = %L', similar_group_table, visable_count, visable_count_drive, similar_group_id);
END
$$;
CREATE OR REPLACE FUNCTION public.update_similar_group_cover(schema text, similar_group_id integer)
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  cover text;
BEGIN
  similar_group_table = schema || '.similar_group';
  many_similar_group_has_many_item_table = schema || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT id_item from unit WHERE id_item in (SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item) ORDER BY place limit 1', many_similar_group_has_many_item_table, similar_group_id) INTO cover;
  IF (cover IS NULL) THEN
    cover = 0;
  END IF;
  EXECUTE format('UPDATE %s SET cover = %L WHERE id = %L', similar_group_table, cover, similar_group_id);
END
$$;
CREATE OR REPLACE FUNCTION public.update_unit_place_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  many_similar_group_has_many_item_table text;
  similar_group_id bigint;
  id_array bigint[];
BEGIN
  -- place = 1: in drive
  IF (NEW.place <> 1 AND OLD.place <> 1) THEN
    RETURN NULL;
  END IF;
  many_similar_group_has_many_item_table = TG_TABLE_SCHEMA || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT array_agg(DISTINCT id_similar_group) FROM %s WHERE id_item = %L', many_similar_group_has_many_item_table, NEW.id_item) INTO id_array;
  IF (id_array IS NULL) THEN
    RETURN NULL;
  END IF;
  FOREACH similar_group_id IN ARRAY id_array LOOP
    EXECUTE format('SELECT public.update_similar_group_cover(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
    EXECUTE format('SELECT public.update_similar_group_count(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  END LOOP;
  RETURN NULL;
END;
$$;
CREATE TRIGGER update_similar_group_after_update_unit_place_trigger
  AFTER UPDATE OF place
  ON public.unit
  FOR EACH ROW
  EXECUTE PROCEDURE public.update_unit_place_func();
CREATE OR REPLACE FUNCTION public.update_similar_info_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  cover text;
  current_cover bigint;
  item_count bigint;
  similar_group_id bigint;
  need_update_cover bool;
BEGIN
  similar_group_table = TG_TABLE_SCHEMA || '.similar_group';
  many_similar_group_has_many_item_table = TG_TABLE_SCHEMA || '.many_similar_group_has_many_item';
  need_update_cover = false;
  IF (TG_OP = 'DELETE') THEN
    similar_group_id = OLD.id_similar_group;
  ELSE
    similar_group_id = NEW.id_similar_group;
  END IF;
  EXECUTE format('SELECT count(*) FROM %s WHERE id_similar_group = %L', many_similar_group_has_many_item_table, similar_group_id) INTO item_count;
  EXECUTE format('SELECT cover FROM %s WHERE id = %L', similar_group_table, similar_group_id) INTO current_cover;
  IF (TG_OP = 'INSERT' AND current_cover IS NULL) THEN
    need_update_cover = true;
  ELSIF (TG_OP = 'UPDATE' AND current_cover = NEW.id_item AND NEW.hidden = 't') THEN
    need_update_cover = true;
  ELSIF (TG_OP = 'DELETE' AND current_cover = OLD.id_item) THEN
    need_update_cover = true;
  END IF;
  IF (item_count IS NULL) THEN
    EXECUTE format('DELETE FROM %s WHERE id = %L', similar_group_table, similar_group_id);
    RETURN NULL;
  END IF;
  IF (need_update_cover OR current_cover = 0) THEN
    EXECUTE format('SELECT public.update_similar_group_cover(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  END IF;
  EXECUTE format('SELECT public.update_similar_group_count(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  RETURN NULL;
END
$$;