CREATE OR REPLACE FUNCTION public.update_unit_place_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  unit_table text;
  folder_table text;
  many_similar_group_has_many_item_table text;
  similar_group_id bigint;
  id_array bigint[];
  has_moments bigint;
BEGIN
  -- place = 1: in drive;  place = 0: in moments
  IF (NEW.place = OLD.place) THEN
    RETURN NULL;
  END IF;
  IF (NEW.place <> -1) THEN
    unit_table = TG_TABLE_SCHEMA || '.unit';
    folder_table = TG_TABLE_SCHEMA || '.folder';
    EXECUTE format('SELECT count(*) FROM %s WHERE id IN (SELECT id_folder FROM %s WHERE id_item = %L) AND (name LIKE ''%s'' OR name = ''/Moments'')', folder_table, unit_table, NEW.id_item, '/Moments/%') INTO has_moments;
    IF (has_moments > 0) THEN
      EXECUTE format('UPDATE %s SET place = 0 WHERE id_item = %L', unit_table, NEW.id_item);
    ELSE
      EXECUTE format('UPDATE %s SET place = 1 WHERE id_item = %L', unit_table, NEW.id_item);
    END IF;
  END IF;
  IF (NEW.place <> 1 AND OLD.place <> 1) THEN
    RETURN NULL;
  END IF;
  many_similar_group_has_many_item_table = TG_TABLE_SCHEMA || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT array_agg(DISTINCT id_similar_group) FROM %s WHERE id_item = %L', many_similar_group_has_many_item_table, NEW.id_item) INTO id_array;
  IF (id_array IS NULL) THEN
    RETURN NULL;
  END IF;
  FOREACH similar_group_id IN ARRAY id_array LOOP
    EXECUTE format('SELECT public.update_similar_group_cover(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
    EXECUTE format('SELECT public.update_similar_group_count(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  END LOOP;
  RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.unit_update_id_item_func ( schema text, item_id integer, new_item_id integer )
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  many_item_has_many_normal_album_table text;
  many_similar_group_has_many_item_table text;
  album_array bigint[];
  album_id bigint;
  similar_group_array bigint[];
  similar_group bigint;
  record RECORD;
BEGIN
  many_item_has_many_normal_album_table = schema || '.many_item_has_many_normal_album';
  many_similar_group_has_many_item_table = schema || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT array_agg(id_normal_album) FROM %s WHERE id_item = %L', many_item_has_many_normal_album_table, item_id) INTO album_array;
  IF (album_array IS NOT NULL) THEN
    FOREACH album_id IN ARRAY album_array LOOP
      EXECUTE format('SELECT * FROM %s WHERE id_normal_album = %L AND id_item = %L', many_item_has_many_normal_album_table, album_id, new_item_id) INTO record;
      IF (record IS NULL) THEN
        EXECUTE format('UPDATE %s SET id_item = %L WHERE id_normal_album = %L AND id_item = %L', many_item_has_many_normal_album_table, new_item_id, album_id, item_id);
      END IF;
    END LOOP;
  END IF;
  EXECUTE format('SELECT array_agg(id_similar_group) FROM %s WHERE id_item = %L', many_similar_group_has_many_item_table, item_id) INTO similar_group_array;
  IF (similar_group_array IS NOT NULL) THEN
    FOREACH similar_group IN ARRAY similar_group_array LOOP
      EXECUTE format('SELECT * FROM %s WHERE id_similar_group = %L AND id_item = %L', many_similar_group_has_many_item_table, similar_group, new_item_id) INTO record;
      IF (record IS NULL) THEN
        EXECUTE format('UPDATE %s SET id_item = %L WHERE id_similar_group = %L AND id_item = %L', many_similar_group_has_many_item_table, new_item_id, similar_group, item_id);
      END IF;
    END LOOP;
  END IF;
END
$$;
CREATE OR REPLACE FUNCTION public.update_normal_album_time_trigger_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  id_item integer;
  many_table text;
  normal_album_table text;
  max_time bigint;
  min_time bigint;
  record RECORD;
BEGIN
  id_item  = OLD.id_item;
  normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
  many_table = TG_TABLE_SCHEMA || '.many_item_has_many_normal_album';
  FOR record IN EXECUTE format('SELECT * FROM %s WHERE id_item = %L', many_table, id_item) LOOP
    EXECUTE format('SELECT COALESCE(max, 0), COALESCE(min, 0) FROM public.get_unit_takentime_by_album(%L, %L)', record.id_normal_album, TG_TABLE_SCHEMA) INTO max_time, min_time;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || min_time || ', end_time = ' || max_time || ' WHERE id = ' || record.id_normal_album;
  END LOOP;
  RETURN NULL;
END;
$$;