CREATE OR REPLACE FUNCTION public.get_unit_takentime_by_album(id_album int4, schema text)
  RETURNS TABLE (max bigint, min bigint)
  LANGUAGE plpgsql
  STABLE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  unit_table text;
  many_table text;
BEGIN
    unit_table = schema || '.unit';
    many_table = schema || '.many_item_has_many_normal_album';
    RETURN QUERY EXECUTE 'SELECT MAX(takentime), MIN(takentime) FROM ' || unit_table || ' WHERE id_item IN(
        SELECT id_item FROM '|| many_table || ' WHERE id_normal_album = ' || id_album ||')';
END;
$$;
CREATE OR REPLACE FUNCTION public.face_update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  IF (TG_OP = 'UPDATE' AND NEW.id_person != OLD.id_person) THEN
       SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
       EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || NEW.ref_id_unit;
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.delete_sharing_entry ( passphrase text)
  RETURNS bool
  LANGUAGE c
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY DEFINER
  COST 1
  AS '/var/packages/SynologyMoments/target/usr/lib/libsynophoto-db-trigger.so', 'pg_delete_sharing_entry';
CREATE OR REPLACE FUNCTION public.update_normal_album_time_trigger_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  id_item integer;
  many_table text;
  normal_album_table text;
  max_time bigint;
  min_time bigint;
  record RECORD;
BEGIN
  id_item  = OLD.id_item;
  normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
  many_table = TG_TABLE_SCHEMA || '.many_item_has_many_normal_album';
  FOR record IN EXECUTE format('SELECT * FROM %s WHERE id_item = %L', many_table, id_item) LOOP
    EXECUTE format('SELECT COALESCE(max, 0), COALESCE(min, 0) FROM public.get_unit_takentime_by_album(%L, %L)', record.id_normal_album, TG_TABLE_SCHEMA) INTO max_time, min_time;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || min_time || ', end_time = ' || max_time || ' WHERE id = ' || record.id_normal_album;
  END LOOP;
  RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.address_to_tsvector_immutable_func ( address json)
	RETURNS tsvector
	LANGUAGE plpgsql
	IMMUTABLE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    result tsvector;
    field text;
    value text;
    address_array text[];
    geo text[] = '{
      "country",
      "state",
      "county",
      "city",
      "town",
      "district",
      "village",
      "route",
      "landmark"
    }';
BEGIN
    FOREACH field IN ARRAY geo
    LOOP
        SELECT address::json->field, array_append(address_array, value) INTO value, address_array;
    END LOOP;
    SELECT to_tsvector(array_to_string(address_array, ' ')) INTO result;
    RETURN result;
END
$$;
CREATE OR REPLACE FUNCTION public.update_face_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  custom_cover bool;
  cover integer;
  BEGIN
    EXECUTE format('SELECT cover FROM %I.person WHERE id = %s', TG_TABLE_SCHEMA, OLD.id_person) INTO cover;
    IF (cover = OLD.id) THEN
      PERFORM public.update_person_best_cover(OLD.id_person, TG_TABLE_SCHEMA);
    END IF;
    RETURN NULL;
  END;
$$;
CREATE OR REPLACE FUNCTION public.update_user_schema( old_version integer, new_version integer)
	RETURNS text
	LANGUAGE c
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY DEFINER
	COST 1
	AS '/var/packages/SynologyMoments/target/usr/lib/libsynophoto-db-trigger.so', 'update_user_schema';
CREATE OR REPLACE FUNCTION public.update_normal_album_cover_func ( schema text,  album_id integer)
	RETURNS void
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    many_item_has_many_normal_album_table text;
    normal_album_table  text;
    item_id integer;
BEGIN
    many_item_has_many_normal_album_table = schema || '.many_item_has_many_normal_album';
    normal_album_table = schema || '.normal_album';
    EXECUTE format('SELECT id_item FROM %s WHERE id_normal_album = %L', many_item_has_many_normal_album_table, album_id) INTO item_id;
    IF item_id IS NULL THEN
      item_id = 0;
    END IF;
    EXECUTE format('UPDATE %s SET cover = %L WHERE id = %L', normal_album_table, item_id, album_id);
END
$$;
CREATE OR REPLACE FUNCTION public.update_similar_group_count(schema text, similar_group_id integer)
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  unit_table text;
  visable_count bigint;
  visable_count_drive bigint;
BEGIN
  similar_group_table = schema || '.similar_group';
  many_similar_group_has_many_item_table = schema || '.many_similar_group_has_many_item';
  unit_table = schema || '.unit';
  EXECUTE format('SELECT COUNT(DISTINCT(id_item)) from %s WHERE id_item in (SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item) AND place <= 0', unit_table, many_similar_group_has_many_item_table, similar_group_id) INTO visable_count;
  IF (visable_count IS NULL) THEN
    visable_count = 0;
  END IF;
  EXECUTE format('SELECT COUNT(DISTINCT(id_item)) from %s WHERE id_item in (SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item) AND place <= 1', unit_table, many_similar_group_has_many_item_table, similar_group_id) INTO visable_count_drive;
  IF (visable_count_drive IS NULL) THEN
    visable_count_drive = 0;
  END IF;
  EXECUTE format('UPDATE %s SET item_count = %L, item_count_drive = %L WHERE id = %L', similar_group_table, visable_count, visable_count_drive, similar_group_id);
END
$$;
CREATE OR REPLACE FUNCTION public.update_normal_album_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  next_version bigint;
BEGIN
  IF (TG_OP = 'INSERT' or (TG_OP = 'UPDATE' AND (NEW.item_count != OLD.item_count OR NEW.cover != OLD.cover OR NEW.name != OLD.name OR NEW.shared != OLD.shared OR NEW.sort_type != OLD.sort_type OR NEW.passphrase_share != OLD.passphrase_share))) THEN
       SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO next_version;
       NEW.version = next_version;
  END IF;
  RETURN NEW;
END
$$;
CREATE OR REPLACE FUNCTION public.increase_normal_album_version_func ( schema text,  album_id integer)
	RETURNS void
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    normal_album_table  text;
    new_version integer;
    item_id integer;
BEGIN
    normal_album_table = schema || '.normal_album';
    SELECT public.get_next_version_func(schema) INTO new_version;
    EXECUTE format('UPDATE %s SET version = %L WHERE id = %L', normal_album_table, new_version, album_id);
END
$$;
CREATE OR REPLACE FUNCTION public.update_similar_group_cover(schema text, similar_group_id integer)
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  cover text;
BEGIN
  similar_group_table = schema || '.similar_group';
  many_similar_group_has_many_item_table = schema || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT id_item from unit WHERE id_item in (SELECT id_item FROM %s WHERE id_similar_group = %L AND hidden = false ORDER BY id_item) ORDER BY place limit 1', many_similar_group_has_many_item_table, similar_group_id) INTO cover;
  IF (cover IS NULL) THEN
    cover = 0;
  END IF;
  EXECUTE format('UPDATE %s SET cover = %L WHERE id = %L', similar_group_table, cover, similar_group_id);
END
$$;
CREATE OR REPLACE FUNCTION public.delete_normal_album_after_trigger_func ()
    RETURNS trigger
    LANGUAGE plpgsql
    VOLATILE
    CALLED ON NULL INPUT
    SECURITY INVOKER
    COST 1
    AS $$
DECLARE
    next_version bigint;
    delete_album_table text;
    share_table text;
    BEGIN
        delete_album_table = TG_TABLE_SCHEMA || '.delete_album';
        share_table = TG_TABLE_SCHEMA || '.share';
        IF (TG_OP = 'DELETE') THEN
           SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO next_version;
           EXECUTE 'INSERT INTO ' || delete_album_table || ' (id_normal_album, version)
            VALUES ( '|| OLD.id || ', ' || next_version || ')';
           IF OLD.passphrase_share IS NOT NULL THEN
               EXECUTE format('DELETE FROM %s WHERE passphrase=%L', share_table, OLD.passphrase_share);
           END IF;
        END IF;
        RETURN NULL;
    END;
$$;
CREATE OR REPLACE FUNCTION public.delete_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
schema  text;
BEGIN
   schema = 'user_' || OLD.id;
   EXECUTE 'DROP SCHEMA ' || schema || ' CASCADE';
   RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.tag_update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
  IF (TG_OP = 'INSERT') THEN
     EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || NEW.id_unit;
  ELSIF (TG_OP = 'DELETE') THEN
     EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || OLD.id_unit;
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.item_type_update_unit_version_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  IF (TG_OP = 'UPDATE') THEN
    SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
    EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id_item = ' || NEW.id;
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.insert_item_to_udpate_normal_album_func(id_album int4, schema text)
  RETURNS VOID
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  many_table text;
  normal_album_table text;
  max_time bigint;
  min_time bigint;
  count integer;
BEGIN
  normal_album_table = schema || '.normal_album';
  many_table = schema || '.many_item_has_many_normal_album';
  EXECUTE 'SELECT count(*) FROM ' || many_table || ' WHERE id_normal_album = ' || id_album INTO count;
  EXECUTE format('SELECT COALESCE(max, 0), COALESCE(min, 0) FROM public.get_unit_takentime_by_album(%L, %L)', id_album, schema) INTO max_time, min_time;
  EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || min_time || ', end_time = ' || max_time || ', item_count = ' || count || ' WHERE id = ' || id_album;
END;
$$;
CREATE OR REPLACE FUNCTION public.update_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  schema text;
  schema_sql text;
  version integer;
  old_version integer;
BEGIN
  schema = 'user_' || NEW.id;
  IF (NEW.id != OLD.id) THEN
    EXECUTE format('ALTER SCHEMA user_%s RENAME TO user_%s', OLD.id, NEW.id);
  END IF;
  version = NEW.config->'version';
  old_version = OLD.config->'version';
  SELECT COALESCE(old_version, 1) INTO old_version;
  SELECT COALESCE(version, 1) INTO version;
  IF (old_version != version) THEN
    SELECT update_user_schema(old_version, version) INTO schema_sql;
    IF schema_sql != '' THEN
      EXECUTE format('SET LOCAL search_path TO %L, %L', 'user_' || NEW.id, 'public');
      EXECUTE schema_sql;
      EXECUTE format('SET LOCAL search_path TO %L', 'public');
    END IF;
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.update_burst_additional_sequence_func()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  current_sequence smallint;
  count smallint;
  burst_additional text;
BEGIN
  burst_additional = TG_TABLE_SCHEMA || '.burst_additional';
  EXECUTE format('SELECT count(*) FROM %s WHERE grouping_key = %L', burst_additional, NEW.grouping_key) INTO count;
  IF (count > 0) THEN
    EXECUTE format('SELECT sequence FROM %s WHERE grouping_key = %L ORDER BY sequence DESC limit 1', burst_additional, NEW.grouping_key) INTO current_sequence;
    NEW.sequence = current_sequence + 1;
  ELSE
    NEW.sequence = 1;
  END IF;
  RETURN NEW;
END
$$;
CREATE OR REPLACE FUNCTION public.update_normal_album_trigger_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
    album_id integer;
    item_id integer;
    is_delete bool;
    album_start_time bigint;
    album_end_time bigint;
    unit_time bigint;
    unit_table text;
    normal_album_table text;
    album_takentime_max bigint;
    album_takentime_min bigint;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
  IF (TG_OP = 'INSERT') THEN
    album_id = NEW.id_normal_album;
    item_id  = NEW.id_item;
    is_delete= false;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) + 1 WHERE id =' || album_id;
  ELSIF (TG_OP = 'DELETE') THEN
    album_id = OLD.id_normal_album;
    item_id  = OLD.id_item;
    is_delete= true;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) - 1 WHERE id =' || album_id;
  END IF;
  EXECUTE 'SELECT takentime FROM ' || unit_table || ' WHERE id_item = ' || item_id INTO unit_time;
  EXECUTE 'SELECT COALESCE(start_time, 0), COALESCE(end_time, 0) FROM ' || normal_album_table ||
          ' WHERE id =' || album_id INTO album_start_time, album_end_time;
  IF (unit_time IS NULL OR (is_delete AND (unit_time = album_start_time OR unit_time = album_end_time))) THEN
    EXECUTE format('SELECT COALESCE(max, 0), COALESCE(min, 0) FROM public.get_unit_takentime_by_album(%L, %L)', album_id, TG_TABLE_SCHEMA) INTO album_takentime_max, album_takentime_min;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || album_takentime_min || ', end_time = ' || album_takentime_max || ' WHERE id = ' || album_id;
  ELSE
    IF (album_start_time = 0 OR unit_time < album_start_time) THEN
      EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || unit_time || ' WHERE id = ' || album_id;
    END IF;
    IF (album_end_time = 0 OR unit_time > album_end_time) THEN
      EXECUTE 'UPDATE ' || normal_album_table || ' SET end_time = ' || unit_time || ' WHERE id = ' || album_id;
    END IF;
  END IF;
  RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.metadata_update_unit_version_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  new_version bigint;
  unit_table text;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  IF (TG_OP = 'UPDATE') THEN
    IF (NEW.description != OLD.description OR NEW.orientation != OLD.orientation) THEN
       SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
       EXECUTE 'UPDATE ' || unit_table || ' SET version = ' || new_version || ' WHERE id = ' || NEW.id_unit;
    END IF;
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.generate_user_schema ( version integer)
  RETURNS text
  LANGUAGE c
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY DEFINER
  COST 1
  AS '/var/packages/SynologyMoments/target/usr/lib/libsynophoto-db-trigger.so', 'generate_user_schema';
CREATE OR REPLACE FUNCTION public.delete_expired_version_time_data_func()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  count bigint;
  version_time text;
BEGIN
  version_time = TG_TABLE_SCHEMA || '.version_time';
  EXECUTE format('SELECT count(*) FROM %s', version_time) INTO count;
  IF (count > 110000) THEN
		EXECUTE format('DELETE FROM %s WHERE version IN (SELECT version FROM %s order by version limit %L)', version_time, version_time, 10000);
  END IF;
	RETURN NEW;
END
$$;
CREATE OR REPLACE FUNCTION public.update_unit_place_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  unit_table text;
  folder_table text;
  many_similar_group_has_many_item_table text;
  similar_group_id bigint;
  id_array bigint[];
  has_moments bigint;
BEGIN
  -- place = 1: in drive;  place = 0: in moments
  IF (NEW.place = OLD.place) THEN
    RETURN NULL;
  END IF;
  IF (NEW.place <> -1) THEN
    unit_table = TG_TABLE_SCHEMA || '.unit';
    folder_table = TG_TABLE_SCHEMA || '.folder';
    EXECUTE format('SELECT count(*) FROM %s WHERE id IN (SELECT id_folder FROM %s WHERE id_item = %L) AND (name LIKE ''%s'' OR name = ''/Moments'')', folder_table, unit_table, NEW.id_item, '/Moments/%') INTO has_moments;
    IF (has_moments > 0) THEN
      EXECUTE format('UPDATE %s SET place = 0 WHERE id_item = %L', unit_table, NEW.id_item);
    ELSE
      EXECUTE format('UPDATE %s SET place = 1 WHERE id_item = %L', unit_table, NEW.id_item);
    END IF;
  END IF;
  IF (NEW.place <> 1 AND OLD.place <> 1) THEN
    RETURN NULL;
  END IF;
  many_similar_group_has_many_item_table = TG_TABLE_SCHEMA || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT array_agg(DISTINCT id_similar_group) FROM %s WHERE id_item = %L', many_similar_group_has_many_item_table, NEW.id_item) INTO id_array;
  IF (id_array IS NULL) THEN
    RETURN NULL;
  END IF;
  FOREACH similar_group_id IN ARRAY id_array LOOP
    EXECUTE format('SELECT public.update_similar_group_cover(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
    EXECUTE format('SELECT public.update_similar_group_count(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  END LOOP;
  RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.delete_share_cascade_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
  DECLARE
    key text;
    accessed_share_album text;
    delete_success bool;
  BEGIN
    accessed_share_album = TG_TABLE_SCHEMA || '.accessed_share_album';
    key = OLD.passphrase;
    EXECUTE 'DELETE FROM ' || accessed_share_album || ' WHERE passphrase = ' || quote_literal(key);
    SELECT delete_sharing_entry(key) INTO delete_success;
    IF (delete_success = false) THEN
      RAISE EXCEPTION 'delete synoshareing entry failed';
    END IF;
    RETURN NULL;
  END
$$;
CREATE OR REPLACE FUNCTION public.create_delete_team_user_trigger_func ()
    RETURNS trigger
    LANGUAGE plpgsql
    VOLATILE
    CALLED ON NULL INPUT
    SECURITY INVOKER
    COST 1
    AS $$
DECLARE
  version integer;
  name text;
  id integer;
BEGIN
  EXECUTE format('SELECT value FROM config WHERE key = %L', 'version') INTO version;
  IF (TG_OP = 'INSERT') THEN
    EXECUTE format('INSERT INTO user_info (name, config, enable) VALUES (%L, \'{"version"%s %s}\', true) RETURNING id', NEW.path, ':', version) INTO id;
    EXECUTE format('DELETE FROM user_%s.folder WHERE id = 1', id);
    EXECUTE format('INSERT INTO user_flag (id_user_info, flag, value) VALUES (%L, \'update_file_mtime_done\', \'true\')', id);
  ELSEIF (TG_OP = 'UPDATE') THEN
    EXECUTE format('UPDATE user_info SET name = %L WHERE name = %L', NEW.path, OLD.path);
  ELSEIF (TG_OP = 'DELETE') THEN
    EXECUTE format('DELETE FROM user_info WHERE name = %L AND uid IS NULL', OLD.path);
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.delete_nonexisting_geocoding_after_delete_unit()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  geocoding_id integer;
  geocoding_table text;
  unit_count integer;
  unit_table text;
BEGIN
  IF (OLD.id_geocoding IS NULL) THEN
    RETURN OLD;
  END IF;
  unit_table = TG_TABLE_SCHEMA || '.unit';
  geocoding_table = TG_TABLE_SCHEMA || '.geocoding';
  geocoding_id = OLD.id_geocoding;
  EXECUTE 'SELECT COUNT(*) FROM ' || unit_table || ' WHERE id_geocoding = ' || geocoding_id INTO unit_count;
  IF (0 = unit_count) THEN
    EXECUTE 'DELETE FROM ' || geocoding_table || ' WHERE id = ' || geocoding_id;
  END IF;
  RETURN OLD;
END;
$$;
CREATE OR REPLACE FUNCTION public.update_unit_version_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
    new_version bigint;
BEGIN
    IF (TG_OP = 'INSERT') THEN
        SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
        NEW.version = new_version;
        RETURN NEW;
    ELSIF (TG_OP = 'UPDATE') THEN
        IF (NEW.index_stage != OLD.index_stage OR
            NEW.filename != OLD.filename OR
            NEW.id_folder != OLD.id_folder OR
            NEW.place != OLD.place OR
            NEW.takentime != OLD.takentime OR
            NEW.id_geocoding != OLD.id_geocoding OR
            NEW.mtime != OLD.mtime OR
            NEW.cache_key != OLD.cache_key) THEN
           SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
           NEW.version = new_version;
        END IF;
        RETURN NEW;
    END IF;
    RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.accessed_share_album_rotate_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    count integer;
    atime bigint;
    accessed_share_album text;
BEGIN
    accessed_share_album = TG_TABLE_SCHEMA || '.accessed_share_album';
    EXECUTE 'SELECT count(*) FROM '|| accessed_share_album INTO count;
    IF (count > 500) THEN
          EXECUTE 'SELECT access_time FROM '|| accessed_share_album ||'  ORDER BY access_time DESC OFFSET 500 LIMIT 1 ' INTO atime;
          EXECUTE 'DELETE FROM '|| accessed_share_album || ' WHERE access_time <= ' || atime;
    END IF;
    RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.unit_update_id_item_func ( schema text, item_id integer, new_item_id integer )
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  many_item_has_many_normal_album_table text;
  many_similar_group_has_many_item_table text;
  album_array bigint[];
  album_id bigint;
  similar_group_array bigint[];
  similar_group bigint;
  record RECORD;
BEGIN
  many_item_has_many_normal_album_table = schema || '.many_item_has_many_normal_album';
  many_similar_group_has_many_item_table = schema || '.many_similar_group_has_many_item';
  EXECUTE format('SELECT array_agg(id_normal_album) FROM %s WHERE id_item = %L', many_item_has_many_normal_album_table, item_id) INTO album_array;
  IF (album_array IS NOT NULL) THEN
    FOREACH album_id IN ARRAY album_array LOOP
      EXECUTE format('SELECT * FROM %s WHERE id_normal_album = %L AND id_item = %L', many_item_has_many_normal_album_table, album_id, new_item_id) INTO record;
      IF (record IS NULL) THEN
        EXECUTE format('UPDATE %s SET id_item = %L WHERE id_normal_album = %L AND id_item = %L', many_item_has_many_normal_album_table, new_item_id, album_id, item_id);
      END IF;
    END LOOP;
  END IF;
  EXECUTE format('SELECT array_agg(id_similar_group) FROM %s WHERE id_item = %L', many_similar_group_has_many_item_table, item_id) INTO similar_group_array;
  IF (similar_group_array IS NOT NULL) THEN
    FOREACH similar_group IN ARRAY similar_group_array LOOP
      EXECUTE format('SELECT * FROM %s WHERE id_similar_group = %L AND id_item = %L', many_similar_group_has_many_item_table, similar_group, new_item_id) INTO record;
      IF (record IS NULL) THEN
        EXECUTE format('UPDATE %s SET id_item = %L WHERE id_similar_group = %L AND id_item = %L', many_similar_group_has_many_item_table, new_item_id, similar_group, item_id);
      END IF;
    END LOOP;
  END IF;
END
$$;
CREATE OR REPLACE FUNCTION public.update_person_best_cover(id_person integer, target_table text)
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  cover integer;
  BEGIN
    EXECUTE format('SELECT id FROM %I.face WHERE id_person = %s and id_unit != 0 ORDER BY score DESC LIMIT 1',
      target_table, id_person) INTO cover;
    IF (cover IS NOT NULL) THEN
      EXECUTE format('UPDATE %I.person SET cover = %s WHERE id = %s', target_table, cover, id_person);
    END IF;
  END;
$$;
CREATE OR REPLACE FUNCTION public.delete_item_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  delete_item_table text;
  new_version bigint;
BEGIN
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
  EXECUTE format('INSERT INTO %s (id_item, version) VALUES (%L, %L)', delete_item_table, OLD.id, new_version);
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.delete_unit_version_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  count integer;
  new_version bigint;
  unit_table text;
  delete_item_table text;
  id_item integer;
  id_user integer;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  delete_item_table = TG_TABLE_SCHEMA || '.delete_item';
  IF (TG_OP = 'DELETE') THEN
    id_item = OLD.id_item;
    EXECUTE format('SELECT count(*) FROM %s WHERE id_item = %L', unit_table, id_item) INTO count;
    SELECT public.get_next_version_func(TG_TABLE_SCHEMA) INTO new_version;
    IF (count > 0) THEN
      EXECUTE format('UPDATE %s SET version = %L  WHERE id_item = %L', unit_table, new_version, id_item);
    END IF;
    EXECUTE format('SELECT split_part(%L, %L, 2)', TG_TABLE_SCHEMA, '_') into id_user;
    EXECUTE format('DELETE FROM index_queue WHERE id_user = %L and id_unit = %L', id_user, OLD.id);
  ELSIF (TG_OP = 'UPDATE' AND OLD.reindex_flag = NEW.reindex_flag) THEN
    id_item = OLD.id_item;
    IF (NEW.id_item != OLD.id_item) THEN
      EXECUTE format('SELECT public.unit_update_id_item_func(%L, %L, %L)', TG_TABLE_SCHEMA, id_item, NEW.id_item);
    END IF;
  END IF;
  IF id_item IS NOT NULL THEN
    EXECUTE format('SELECT public.unit_update_normal_album_func(%L, %L)', TG_TABLE_SCHEMA, id_item);
  END IF;
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.update_similar_info_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  AS $$
DECLARE
  similar_group_table text;
  many_similar_group_has_many_item_table text;
  cover text;
  current_cover bigint;
  item_count bigint;
  similar_group_id bigint;
  need_update_cover bool;
BEGIN
  similar_group_table = TG_TABLE_SCHEMA || '.similar_group';
  many_similar_group_has_many_item_table = TG_TABLE_SCHEMA || '.many_similar_group_has_many_item';
  need_update_cover = false;
  IF (TG_OP = 'DELETE') THEN
    similar_group_id = OLD.id_similar_group;
  ELSE
    similar_group_id = NEW.id_similar_group;
  END IF;
  EXECUTE format('SELECT count(*) FROM %s WHERE id_similar_group = %L', many_similar_group_has_many_item_table, similar_group_id) INTO item_count;
  EXECUTE format('SELECT cover FROM %s WHERE id = %L', similar_group_table, similar_group_id) INTO current_cover;
  IF (TG_OP = 'INSERT' AND current_cover IS NULL) THEN
    need_update_cover = true;
  ELSIF (TG_OP = 'UPDATE' AND current_cover = NEW.id_item AND NEW.hidden = 't') THEN
    need_update_cover = true;
  ELSIF (TG_OP = 'DELETE' AND current_cover = OLD.id_item) THEN
    need_update_cover = true;
  END IF;
  IF (item_count IS NULL) THEN
    EXECUTE format('DELETE FROM %s WHERE id = %L', similar_group_table, similar_group_id);
    RETURN NULL;
  END IF;
  IF (need_update_cover OR current_cover = 0) THEN
    EXECUTE format('SELECT public.update_similar_group_cover(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  END IF;
  EXECUTE format('SELECT public.update_similar_group_count(%L, %s)', TG_TABLE_SCHEMA, similar_group_id);
  RETURN NULL;
END
$$;
CREATE OR REPLACE FUNCTION public.notification_rotate_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
    count integer;
    ctime bigint;
	notification_table text;
BEGIN
	notification_table = TG_TABLE_SCHEMA || '.notification';
    EXECUTE 'SELECT count(*) FROM ' || notification_table INTO count;
    IF (count > 200) THEN
          EXECUTE 'SELECT createtime FROM ' || notification_table || ' ORDER BY createtime DESC OFFSET 200 LIMIT 1' INTO ctime;
          EXECUTE 'DELETE FROM ' || notification_table || ' WHERE createtime <= ' || ctime;
    END IF;
    RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.delete_face_trigger_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  face_id integer;
  person_id integer;
  group_id integer;
  cluster_id integer;
  group_weight integer;
  remain_group_count integer;
  remain_cluster_count integer;
  person_group_table text;
  cluster_table text;
  person_table text;
  face_table text;
  cover integer;
  BEGIN
    person_group_table = TG_TABLE_SCHEMA || '.person_group';
    person_table = TG_TABLE_SCHEMA || '.person';
    cluster_table = TG_TABLE_SCHEMA || '.cluster';
    face_table = TG_TABLE_SCHEMA || '.face';
    person_id = OLD.id_person;
    group_id = OLD.id_person_group;
    face_id = OLD.id;
    IF (person_id IS NOT NULL) THEN
      EXECUTE format('SELECT cover FROM %I.person WHERE id = %s', TG_TABLE_SCHEMA, person_id) INTO cover;
      IF (cover = OLD.id) THEN
        PERFORM public.update_person_best_cover(person_id, TG_TABLE_SCHEMA);
      END IF;
    END IF;
    IF (group_id IS NOT NULL) THEN
      EXECUTE 'SELECT COUNT(*) FROM ' || face_table || ' WHERE id_person_group = ' || group_id INTO group_weight;
      IF (group_weight > 0) THEN
        RETURN NULL;
      END IF;
      EXECUTE 'SELECT id_cluster FROM ' || person_group_table || ' WHERE id = ' || group_id INTO cluster_id;
      EXECUTE 'DELETE FROM ' || person_group_table || ' WHERE id = ' || group_id;
      IF (cluster_id IS NOT NULL) THEN
        EXECUTE 'SELECT COUNT(*) FROM ' || person_group_table || ' WHERE id_cluster = ' || cluster_id INTO remain_group_count;
        IF (remain_group_count > 0) THEN
          RETURN NULL;
        END IF;
        EXECUTE 'DELETE FROM ' || cluster_table || ' WHERE id = ' || cluster_id;
      END IF;
    END IF;
    IF (person_id IS NOT NULL) THEN
      EXECUTE 'SELECT COUNT(*) FROM ' || cluster_table || ' WHERE id_person = ' || person_id INTO remain_cluster_count;
      IF (remain_cluster_count > 0) THEN
        RETURN NULL;
      END IF;
      EXECUTE 'DELETE FROM ' || person_table || ' WHERE id = ' || person_id;
    END IF;
    RETURN NULL;
  END;
$$;
CREATE OR REPLACE FUNCTION public.unit_update_normal_album_func ( schema text, item_id integer )
  RETURNS void
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
  normal_album_table text;
  unit_table text;
  count integer;
  record RECORD;
BEGIN
  normal_album_table = schema || '.normal_album';
  unit_table = schema || '.unit';
  EXECUTE format('SELECT count(*) FROM %s WHERE id_item = %L', unit_table, item_id) INTO count;
  IF (count > 0) THEN
    FOR record IN EXECUTE format('SELECT * FROM %s WHERE cover = %L', normal_album_table, item_id) LOOP
      EXECUTE format('SELECT public.increase_normal_album_version_func(%L, %s)', schema, record.id);
    END LOOP;
  ELSE
    FOR record IN EXECUTE format('SELECT * FROM %s WHERE cover = %L', normal_album_table, item_id) LOOP
      EXECUTE format('SELECT public.update_normal_album_cover_func(%L, %s, %L)', schema, record.id, item_id);
    END LOOP;
  END IF;
END
$$;
CREATE OR REPLACE FUNCTION public.delete_item_to_update_normal_album_trigger_func()
  RETURNS trigger
  LANGUAGE plpgsql
  VOLATILE
  CALLED ON NULL INPUT
  SECURITY INVOKER
  COST 1
  AS $$
DECLARE
    album_id integer;
    item_id integer;
    album_start_time bigint;
    album_end_time bigint;
    unit_time bigint;
    unit_table text;
    normal_album_table text;
    album_takentime_max bigint;
    album_takentime_min bigint;
BEGIN
  unit_table = TG_TABLE_SCHEMA || '.unit';
  normal_album_table = TG_TABLE_SCHEMA || '.normal_album';
  album_id = OLD.id_normal_album;
  item_id  = OLD.id_item;
  EXECUTE 'UPDATE ' || normal_album_table || ' SET item_count = COALESCE(item_count, 0) - 1 WHERE id =' || album_id;
  EXECUTE 'SELECT takentime FROM ' || unit_table || ' WHERE id_item = ' || item_id INTO unit_time;
  EXECUTE 'SELECT COALESCE(start_time, 0), COALESCE(end_time, 0) FROM ' || normal_album_table ||
          ' WHERE id =' || album_id INTO album_start_time, album_end_time;
  IF (unit_time IS NULL OR unit_time = album_start_time OR unit_time = album_end_time) THEN
    EXECUTE format('SELECT COALESCE(max, 0), COALESCE(min, 0) FROM public.get_unit_takentime_by_album(%L, %L)', album_id, TG_TABLE_SCHEMA) INTO album_takentime_max, album_takentime_min;
    EXECUTE 'UPDATE ' || normal_album_table || ' SET start_time = ' || album_takentime_min || ', end_time = ' || album_takentime_max || ' WHERE id = ' || album_id;
  END IF;
  RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.get_next_version_func ( schema text)
	RETURNS bigint
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
  next_version bigint;
  version_table text;
BEGIN
  version_table = schema || '.version_time';
  EXECUTE 'INSERT INTO ' || version_table || '(modified_time)
    SELECT round(extract(epoch FROM now()) * 1000) RETURNING version ' INTO next_version;
  RETURN next_version;
END
$$;
CREATE OR REPLACE FUNCTION public.update_normal_album_cover_func ( schema text, album_id integer, old_item_id integer )
RETURNS void
LANGUAGE plpgsql
VOLATILE
CALLED ON NULL INPUT
SECURITY INVOKER
COST 1
AS $$
DECLARE
many_item_has_many_normal_album_table text;
normal_album_table  text;
item_id integer;
BEGIN
  many_item_has_many_normal_album_table = schema || '.many_item_has_many_normal_album';
  normal_album_table = schema || '.normal_album';
  EXECUTE format('SELECT id_item FROM %s WHERE id_normal_album = %L AND id_item != %L', many_item_has_many_normal_album_table, album_id, old_item_id) INTO item_id;
  IF item_id IS NULL THEN
    item_id = 0;
  END IF;
  EXECUTE format('UPDATE %s SET cover = %L WHERE id = %L', normal_album_table, item_id, album_id);
END
$$;
CREATE OR REPLACE FUNCTION public.create_user_schema_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
schema_name     text;
schema_sql      text;
t               text;
version         integer;
inherit_table   text[] = '{
    "unit", "video_additional", "live_additional", "burst_additional",
    "item",
    "album", "condition_album", "normal_album",
    "face", "person", "cluster",  "person_group",
    "folder", "general_tag",
    "geocoding", "geocoding_info",  "address",
    "many_share_has_many_group_info",
    "many_share_has_many_user_info",
    "many_unit_has_many_concept",
    "many_unit_has_many_concept_hidden",
    "many_unit_has_many_general_tag",
    "many_item_has_many_normal_album",
    "metadata",
    "share", "share_comment",
    "thumbnail",
    "version_time",
    "delete_item", "delete_album",
    "notification",
    "thumb_preview",
    "search_timeline",
    "accessed_share_album",
    "video_convert", "assessment", "enhancement", "discover_style",
    "many_similar_group_has_many_item", "unit_similar_hash", "similar_group"}';
BEGIN
    schema_name = 'user_'|| NEW.id;
    EXECUTE format('SELECT value FROM public.config WHERE key=%L', 'version') INTO version;
    EXECUTE format('CREATE SCHEMA %s', schema_name);
    EXECUTE format('SET LOCAL search_path TO %L, %L', schema_name, 'public');
    SELECT public.generate_user_schema(version) INTO schema_sql;
    EXECUTE schema_sql;
    FOREACH t IN ARRAY inherit_table
    LOOP
        EXECUTE format('ALTER TABLE %I INHERIT public.%I', t, t);
    END LOOP;
    EXECUTE format('SET LOCAL search_path TO %L', 'public');
    RETURN NULL;
END;
$$;
CREATE OR REPLACE FUNCTION public.update_tag_ref_count_func ()
	RETURNS trigger
	LANGUAGE plpgsql
	VOLATILE
	CALLED ON NULL INPUT
	SECURITY INVOKER
	COST 1
	AS $$
DECLARE
   tag_id integer;
   check_id integer;
   ref_count integer;
   general_tag_table text;
    BEGIN
        general_tag_table = TG_TABLE_SCHEMA || '.general_tag';
        IF (TG_OP = 'INSERT') THEN
            tag_id = NEW.id_general_tag;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) + 1 WHERE id = ' || tag_id;
        ELSIF (TG_OP = 'UPDATE') THEN
             tag_id = NEW.id_general_tag;
             check_id = OLD.id_general_tag;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) + 1 WHERE id = ' || tag_id;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) - 1 WHERE id = ' || check_id;
        ELSIF (TG_OP = 'DELETE') THEN
            check_id = OLD.id_general_tag;
            EXECUTE 'UPDATE ' || general_tag_table || ' SET count = COALESCE(count, 0) - 1 WHERE id = ' || check_id;
        END IF;
        IF check_id IS NOT NULL THEN
           EXECUTE 'SELECT count  FROM '|| general_tag_table || ' WHERE id = ' || check_id INTO ref_count;
           IF (ref_count  <=0 ) THEN
            EXECUTE 'DELETE FROM ' || general_tag_table || ' WHERE id = ' || check_id;
            END IF;
        END IF;
        RETURN NULL;
    END;
$$;
