#!/usr/bin/env python
#-*- coding: utf-8 -*-
# Copyright (c) 2000-2018 Synology Inc. All rights reserved.

import syslog

from argparse import ArgumentParser

PACKAGE_NAME = 'SynologyMoments'
PACKAGE_PATH = '/var/packages/{}'.format(PACKAGE_NAME)
APP_ID = 'SYNO.Photo.AppInstance'

def log (msg):
    syslog.syslog(syslog.LOG_ERR, str(msg))

class PluginRunner(object):
    def __init__(self):
        self._version = '1.0'

    @property
    def version(self):
        return self._version

    def pre(self):
        pass

    def post(self):
        pass

    def restart_moments_service(self):
        import subprocess
        import time

        subprocess.check_call(['initctl', 'stop', 'pkg-SynologyMoments-apid'])
        subprocess.check_call(['initctl', 'stop', 'pkg-SynologyMoments-check-center'])
        subprocess.check_call(['initctl', 'stop', 'pkg-SynologyMoments-thumbnail-provider'])
        subprocess.check_call(['initctl', 'stop', 'pkg-SynologyMoments-task-center'])
        time.sleep(1)
        subprocess.check_call(['initctl', 'start', 'pkg-SynologyMoments-task-center'])
        subprocess.check_call(['initctl', 'start', 'pkg-SynologyMoments-thumbnail-provider'])
        subprocess.check_call(['initctl', 'start', 'pkg-SynologyMoments-check-center'])
        subprocess.check_call(['initctl', 'start', 'pkg-SynologyMoments-apid'])

    def send_command(self, command, args = None, binary = None):
        self._assertIsAllowedCommand(command)

        import subprocess
        import json
        if binary != None:
          command_path = '{}/target/usr/bin/synophoto-bin-{}'.format(PACKAGE_PATH, binary)
        else:
          command_path = '{}/target/usr/bin/synophoto-bin-sdk-plugin'.format(PACKAGE_PATH)

        arguments = [command_path, '--{}'.format(command)]

        if args != None:
            args_str = json.dumps(args)
            arguments.append(args_str)

        subprocess.check_call(arguments)

    def _assertIsAllowedCommand(self, command):
        allowed_list = [
            'group-delete',
            'group-rename',
            'reload-app-priv',
            'user-enable-change',
            'user-delete',
            'user-rename',
            'share-delete',
            'check-user-group',
            'share-set',
            'user-home-enable-change',
            'share-enable',
            'share-disable',
        ]
        assert command in allowed_list, 'command [{}] is not in allowed list'.format(
            command)

    def is_moments_running(self):
        import subprocess
        res = subprocess.call(['synopkg', 'status', 'SynologyMoments'])

        return res == 0

def MainBody(plugin_instance):
    def print_package_info(key):
        import subprocess
        info = '{}/INFO'.format(PACKAGE_PATH)
        args = ['/bin/get_key_value', info, key]
        output, error = subprocess.Popen(args).communicate()
        print output

    parser = ArgumentParser(description='{} SDK Hooks'.format(PACKAGE_NAME))
    parser.add_argument("--sdk-mod-ver", action="store_true")
    parser.add_argument("--name", action="store_true")
    parser.add_argument("--pkg-ver", action="store_true")
    parser.add_argument("--vendor", action="store_true")
    parser.add_argument("--pre", action="store_true")
    parser.add_argument("--post", action="store_true")
    options = parser.parse_args()

    if options.sdk_mod_ver:
        print plugin_instance.version
    elif options.name:
        print_package_info("package")
    elif options.pkg_ver:
        print_package_info("version")
    elif options.vendor:
        print_package_info("maintainer")
    elif options.pre:
        plugin_instance.pre()
    elif options.post:
        plugin_instance.post()
